 /***************************************************************************\
 * ThyAPI - Thyamad Javascript API - Base Element                            *
 * http://www.thyamad.com                                                    *
 *                                                                           *
 * Copyright (C) 2005 - Raphael Derosso Pereira                              *
 * Based on DynAPI v3.0b1                                                    *
 * ------------------------------------------------------------------------- *
 *  This library is free software; you can redistribute it and/or modify it  *
 *  under the terms of the GNU Lesser General Public License as published    *
 *  by the Free Software Foundation; either version 2.1 of the License,      *
 *  or any later version.                                                    *
 *  This library is distributed in the hope that it will be useful, but      *
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *  See the GNU Lesser General Public License for more details.              *
 \***************************************************************************/

	/**
	 * Class: thyBase
	 *
	 *	Base class for all other ThyAPI Components. It defines fundamental
	 *	functions that all thyAPI components must have.
	 *
	 */
	function thyBase (name)
	{
		this.DynObject = DynObject;
		this.DynObject();
	}

	/**
	 * Method: dynapi.setPrototype
	 *
	 *	Overloaded DynAPI method that permits multiple inheritance
	 *
	 * Parameters:
	 *
	 *	sC - Derivate Class
	 *	sP - Base class (or an array of base classes)
	 *
	 */
	dynapi.setPrototype = function(sC,sP)
	{
		if (typeof(sP) == 'object')
		{
			if ((sP.constructor+'')!=(Array+'')) return alert('Prototype Error');
		}
		else
		{
			sP = [sP];
		}
		
		var i;
		var c = this.frame[sC];
		if (!c) return alert('Prototype Error. Unable to find '+sC);
		
		var p = this.frame[sP[0]];
		if (!p) return alert('Prototype Error. Unable to find '+sP[0]);

		c.prototype = new p();
		
		for (i=1; i<sP.length; i++)
		{
			p = this.frame[sP[i]];
			if (!p) return alert('Prototype Error. Unable to find '+sP[i]);
			var j,prototype = new p();
			for (j in prototype)
			{
				c.prototype[j] = prototype[j];
			}
		}

		c.prototype._className = sC;
		c.prototype._pClassName = sP;
		c.toString = function() {return '['+sC+']'};
		return c.prototype;
	}

	/**
	 * Method: DynObject.isClass
	 *
	 *	Overloaded method that checks if a class is derivate from another class
	 *
	 * Parameters:
	 *
	 *	cn - base class
	 *	n  - derivate class
	 *
	 */
	DynObject.isClass = function(cn,n) {
		if (cn == n) return true;
		else {
			var c = dynapi.frame[cn];
			var p = c.prototype._pClassName;
			if (p)
			{
				if (typeof(p) != 'object') return DynObject.isClass(p,n);
				
				var i, result;
				for (i=0; i<p.length; p++)
				{
					if (DynObject.isClass(p[i],n)) return true;
				}
				
				return false;
			}
			else return false;
		}
	};

	p = dynapi.setPrototype('thyBase', 'DynObject');
	
	/**
	 * Method: isFromClass
	 *
	 *	Checks if the object is from the specified class type or from a class
	 *	derivate from that.
	 *
	 * Parameter:
	 *
	 *	className - The name of the class to be checked
	 *
	 * Returns:
	 *
	 *	Boolean
	 *
	 */
	p.isFromClass = p.isClass;
	
	/**
	 * Method: setName
	 *
	 *	Changes the name of the widget
	 *
	 * Parameter:
	 *
	 *	name - The new widget name
	 *
	 */
	p.setName = function (name)
	{
		if (name == null || this.name == name) return;
		
		this.name = name;
	}

	/**
	 * Method: clone
	 *
	 *	Returns an exact copy of object (including children and DOM nodes)
	 *
	 * Parameter:
	 *
	 *	name - The new name of the object
	 *
	 *	FIXME: NOT DONE YET!!!
	 */
	p.clone = function (name)
	{
		var clone = eval('new '+this._className);
		var i;

		for (i in this)
		{
			switch (typeof(this[i]))
			{
				case 'object':
					// Do not clone special objects
					switch (i)
					{
						case 'parent':
						case 'elm':
						case 'children':
							clone[i] = null;
					}

					if (clone[i] == null) break;
					
					// Clone children
					if (typeof(this[i].clone) == 'function')
					{
						clone[i] = this[i].clone();
						break;
					}
					
				default:
					// Do not clone functions and already cloned nodes
					if (clone[i] == this[i]) break;
					
					clone[i] = dynapi.functions.cloneObject(this[i]);
			}
		}

		if (name)
		{
			clone.setName(name);
		}
		else
		{
			clone.setName(clone.name);
		}

		if (!this._created) return;
		
		var cloneDOM = this.elm.cloneNode(true);
		var updateID = function (elm, children)
		{
			var k;
			for (k=0; k<children.length; k++)
			{
				elm.id = "DynObject"+DynObject._c++;
			}
		}
	}

	//FIXME
	/**
	 * Function: showMessage
	 *
	 *	Shows the message
	 */
	function showMessage(message)
	{
		alert(message);
	}
	
