 /***************************************************************************\
 * ThyAPI - Thyamad Javascript API - Cryptography Element                    *
 * http://thyapi.thyamad.com                                                 *
 *                                                                           *
 * Copyright (C) 2005 - Raphael Derosso Pereira                              *
 * Based on DynAPI v3.0b1                                                    *
 * ------------------------------------------------------------------------- *
 *  This library is free software; you can redistribute it and/or modify it  *
 *  under the terms of the GNU Lesser General Public License as published    *
 *  by the Free Software Foundation; either version 2.1 of the License,      *
 *  or any later version.                                                    *
 *  This library is distributed in the hope that it will be useful, but      *
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *  See the GNU Lesser General Public License for more details.              *
 \***************************************************************************/


	/**
	 * Class: thyCrypt
	 *
	 *	Class that manages cryptography. It allow you to encrypt/decrypt
	 *	things using one of the implemented cyphers. (Now there's just
	 *	blowfish)
	 *
	 */
	function thyCrypt ()
	{
		this.thyBase = thyBase;
		this.thyBase();

		// Attributes \\
		this.cypher = null;
		this.cypherObj = null;
	}

	p = dynapi.setPrototype('thyCrypt', 'thyBase');
	
	/**
	 * Method: setCypher
	 *
	 *	Sets the cypher to be used. Note that this method
	 *	will call library include method, meaning that the
	 *	the cypher will be available only on next run.
	 *
	 *	The only available cypher at the moment is blowfish.
	 *	(Rijndael comming...)
	 *
	 * Parameter:
	 *
	 *	cypher - The cypher type
	 *
	 */
	p.setCypher = function (cypher)
	{
		switch (cypher)
		{
			case 'blowfish':
				dynapi.library.include('thyBlowfish');
				var self = this;
				var create_cypher = function ()
				{
					if (window.thyBlowfish)
					{
						self = new thyBlowfish();
						return;
					}
					setTimeout(create_cypher, 100);
				}
				create_cypher();
				return;
		}
	}
	 
	/**
	 * Method: encode
	 *
	 *	Encodes the text passed with key
	 *
	 * Parameters:
	 *
	 *	str - The text to be encoded
	 *	key - The key to be used to encode the text
	 *
	 * Returns:
	 *
	 *	Base64 encoded string
	 *
	 */
	p.encode = function (str, key)
	{
		throw({
			result:'Cypher not set or decode called before library inclusion. Call on next timeout', 
			location: 'thyCrypt.encode'
		});
	}

	/**
	 * Method: decode
	 *
	 *	Decodes a base64 encoded string to the original
	 *	one, if key is correct
	 *
	 * Parameters:
	 *
	 *	code - The cyphered code
	 *	key  - The key to be used
	 */
	p.decode = function (code, key)
	{
		throw({
			result:'Cypher not set or decode method called before library inclusion. Call on next timeout', 
			location: 'thyCrypt.decode'
		});
	}

	/**
	 * Method: setCustomRandomGenerator
	 *
	 *	Sets the random number generator function to be a custom
	 *	one. If not called, the class will use the default one.
	 *
	 * Parameters:
	 *
	 *	rand - The function to be called
	 *
	 */
	p.setCustomRandomGenerator = function (rand)
	{
	}
