 /*****************************************************************************\
 * ThyAPI - Thyamad Javascript API - DataSource Element                       *
 * http://www.thyamad.com                                                     *
 *                                                                            *
 * Copyright (C) 2005 - Raphael Derosso Pereira                               *
 * Based on DynAPI v3.0b1                                                     *
 * -------------------------------------------------------------------------  *
 *  This library is free software; you can redistribute it and/or modify it   *
 *  under the terms of the GNU Lesser General Public License as published     *
 *  by the Free Software Foundation; either version 2.1 of the License,       *
 *  or any later version.                                                     *
 *  This library is distributed in the hope that it will be useful, but       *
 *  WITHOUT ANY WARRANTY; without even the implied warranty of                *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
 *  See the GNU Lesser General Public License for more details.               *
 \****************************************************************************/

	/**
	 * Class: thyDataSource
	 *
	 *	Class to automate data exchange between client and server
	 *
	 * Events:
	 *
	 *	read
	 *	write 
	 *	startup
	 */
	function thyDataSource(name, startupMessage)
	{
		this.thyBase = thyBase;
		this.thyBase();
		this.DynElement = DynElement;
		this.DynElement();

		// Attributes
		var self = this;
		this.name = name;
		this.serverURL;
		this.serverReadMethod;
		this.serverWriteMethod;
		this.serverRemoveMethod;
		this.serverStartupMethod;
		this.protocol;
		this.started = false;
		this.useSync = false;

		// thyConnector is a Singleton!
		if (!window.thyConnectorObj)
		{
			window.thyConnectorObj = new thyConnector();
			window.thyConnectorObj.setVisible(true);
		}
			
		this.connector = thyConnectorObj;

		this._linkedObj = {read: {}, write: {}, remove:{}, startup: {}};

		// Initialization
		this.connector.setProgressContent('thyDataSource.'+this.name+'.startup', 1, startupMessage);
		this.connector.setProgressContent('thyDataSource.'+this.name+'.startup', 2, startupMessage);
		this.connector.setProgressContent('thyDataSource.'+this.name+'.startup', 3, startupMessage);

		this.addEventListener({
			onstartup: function () { self.started = true; }
		});
	}
	
	p = dynapi.setPrototype('thyDataSource',['thyBase','DynElement']);

	/**
	 * Method: populate
	 *
	 *	Sends the appropriate values to the appropriate linked element
	 *
	 * Parameter:
	 *
	 *	data - The data that should be feed up
	 *
	 *	event - startup, read or write
	 *
	 */
	p.populate = function (data, event)
	{
		var i;
		for (i in data)
		{
			if (!this._linkedObj[event] || 
				!this._linkedObj[event][i] ||
				typeof(this._linkedObj[event][i].populate) != 'function')
			{
				continue;
			}

			this._linkedObj[event][i].populate(data[i]);
		}

		// Invoke event only after finalization, otherwise another
		// call to the same datasource method would fail on thyConnector
		var self = this;
		setTimeout(function () {self.invokeEvent(event, new DynEvent(event, self));});
	}

	/**
	 * Method: setProtocol
	 *
	 *	Sets the protocol to be used by this DataSource
	 *
	 * Parameters:
	 *
	 *	protocol - The Protocol Object aready instantiated
	 */
	p.setProtocol = function (protocol, url)
	{
		if (typeof(protocol) == 'string')
			this.protocol = new thyProtocol('xmlrpc', url);
		else
			this.protocol = protocol;

		this.connector.setContentType(protocol.getContentType());
		this.serverURL = protocol.getServerURL();
	}

	/**
	 * Method: setServerClass
	 *
	 *	If the user of DataSource will communicate to server using
	 *	a standard format, that is, the same class for _startup_, _read_,
	 *	_write_ and _remove_ and that class has these methods, one can call
	 *	<setServerClass> method instead of <setStartupMethod>,
	 *	<setReadMethod>, <setWriteMethod> or <setRemoveMethod>; <setServerClass> 
	 *	will do that, appending the correct string to the parameter passed.
	 *
	 *	- *Note* - This method has been structured to handle call in the
	 *	format 'classname.method', so it will append a '.' and the
	 *	methods to the passed classname.
	 *
	 * Parameter:
	 *
	 *	class - The class name (how it is handled by the server)
	 */
	p.setServerClass = function (className)
	{
		this.setStartupMethod(className+'.startup');
		this.setReadMethod(className+'.read');
		this.setWriteMethod(className+'.write');
		this.setRemoveMethod(className+'.remove');
	}
	
	/**
	 * Method: setWriteMethod
	 *
	 *	Sets the method that will be called when performed a write call
	 *
	 * Parameters:
	 *
	 *	method - The method string
	 *
	 */
	p.setWriteMethod = function (method)
	{
		this.serverWriteMethod = method;
	}

	/**
	 * Method: setReadMethod
	 *
	 *	Sets the method that will be called when performed a read call
	 *
	 * Parameters:
	 *
	 *	method - The method string
	 *
	 */
	p.setReadMethod = function (method)
	{
		this.serverReadMethod = method;
	}

	/**
	 * Method: setRemoveMethod
	 *
	 *	Sets the method that will be called when performed a remove call
	 *
	 * Parameters:
	 *
	 *	method - The method string
	 *
	 */
	p.setRemoveMethod = function (method)
	{
		this.serverRemoveMethod = method;
	}

	/**
	 * Method: setStartupMethod
	 *
	 *	Sets the method that will be called when a startup is performed
	 *
	 * Parameters:
	 *
	 *	method - The method string
	 *
	 */
	p.setStartupMethod = function (method)
	{
		this.serverStartupMethod = method;
	}

	/**
	 * Method: setProgressHolder
	 *
	 *	Tells the Connector to put its message on the specified Progress Holder
	 *
	 * Parameters:
	 *
	 *	holder - The DynLayer (or derivate) that will hold the Messages
	 *
	 */
	p.setProgressHolder = function(holder)
	{
		this.connector.setProgressHolder('thyDataSource.'+this.name+'.startup', holder);
		this.connector.setProgressHolder('thyDataSource.'+this.name+'.read', holder);
		this.connector.setProgressHolder('thyDataSource.'+this.name+'.write', holder);
		this.connector.setProgressHolder('thyDataSource.'+this.name+'.remove', holder);
	}

	/**
	 * Method: setMessageHolder
	 *
	 *	Alias of setProgressHolder
	 *
	 */
	p.setMessageHolder = p.setProgressHolder;

	/**
	 * Method: setReadMessages
	 *
	 *	Tells the Connector which messages are to be shown when calling <read>
	 *
	 * Parameters:
	 *
	 *	messages - Array with messages. There are 3 possible messages:
	 *
	 *		- messages[0] - Contacting Server
	 *		- messages[1] - Server Contacted
	 *		- messages[2] - Loading Server Data
	 *
	 */
	p.setReadMessages = function (messages)
	{
		if (typeof(messages) != 'object') throw({result: 'Invalid Message', location: 'thyDataSource.setReadMessages'});

		this.connector.setProgressContent('thyDataSource.'+this.name+'.read', 1, messages[0]);
		this.connector.setProgressContent('thyDataSource.'+this.name+'.read', 2, messages[1]);
		this.connector.setProgressContent('thyDataSource.'+this.name+'.read', 3, messages[2]);
	}
	
	/**
	 * Method: setWriteMessages
	 *
	 *	Tells the Connector which messages are to be shown when calling <write>
	 *
	 * Parameters:
	 *
	 *	messages - Array with messages. There are 3 possible messages:
	 *
	 *		- messages[0] - Contacting Server
	 *		- messages[1] - Server Contacted
	 *		- messages[2] - Loading Server Data
	 *
	 */
	p.setWriteMessages = function (messages)
	{
		if (typeof(messages) != 'object') throw({result: 'Invalid Message', location: 'thyDataSource.setWriteMessages'});

		this.connector.setProgressContent('thyDataSource.'+this.name+'.write', 1, messages[0]);
		this.connector.setProgressContent('thyDataSource.'+this.name+'.write', 2, messages[1]);
		this.connector.setProgressContent('thyDataSource.'+this.name+'.write', 3, messages[2]);
	}
	
	/**
	 * Method: setRemoveMessages
	 *
	 *	Tells the Connector which messages are to be shown when calling <remove>
	 *
	 * Parameters:
	 *
	 *	messages - Array with messages. There are 3 possible messages:
	 *
	 *		- messages[0] - Contacting Server
	 *		- messages[1] - Server Contacted
	 *		- messages[2] - Loading Server Data
	 *
	 */
	p.setRemoveMessages = function (messages)
	{
		if (typeof(messages) != 'object') throw({result: 'Invalid Message', location: 'thyDataSource.setRemoveMessages'});

		this.connector.setProgressContent('thyDataSource.'+this.name+'.remove', 1, messages[0]);
		this.connector.setProgressContent('thyDataSource.'+this.name+'.remove', 2, messages[1]);
		this.connector.setProgressContent('thyDataSource.'+this.name+'.remove', 3, messages[2]);
	}

	/**
	 * Method: link
	 *
	 *	Link Object to this DataSource
	 *
	 *	Upon calling this method, one attach an element to the instantiated
	 *	DataSource Object on the specified event
	 *
	 * Parameters:
	 *
	 *	obj -  The Object that implements DataLayer methods
	 *	name - The name to be used on data exchange with server
	 *	event - The event (or array of events) that will use this object. Default to all
	 *
	 */
	p.link = function(obj, name, event)
	{
		if (!obj || typeof(obj) != 'object')
		{
			throw({result: 'Invalid Object', location: 'thyDataSource.link'});
		}
		
		if (!name && obj.name) name = obj.name;
		if (!event) event = ['read', 'write', 'remove', 'startup'];
		else if (typeof(event) == 'string') event = [event];
		
		var i;
		for (i=0; i<event.length; i++)
		{
			if (event[i] == 'read' && !(obj.populate && typeof(obj.populate) == 'function'))
			{
				throw({result: 'No populate methods to link on read', location: 'thyDataSource.link'});
			}
			else if (event[i] == 'write' && !(obj.sweepOut && typeof(obj.sweepOut) == 'function'))
			{
				throw({result: 'No populate methods to link on read', location: 'thyDataSource.link'});
			}
			
			this._linkedObj[event[i]][name] = obj;
		}
	}

	/**
	 * Method: unlink
	 *
	 * Unlink Object from this DataSource
	 *
	 *	Upon calling this method, one deattach an element from the instantiated DataSOurce
	 *	Object.
	 *
	 * Parameters:
	 *
	 *	name - The name that was used on data
	 *	event - The event (or array of events) that will not use this object
	 *
	 */
	p.unlink = function(name, event)
	{
		if (!this._linkedObj[name])
			return;
			
		if (!event) event = ['read', 'write', 'remove', 'startup'];
		else if (typeof(event) == 'string') event = [event];
		
		var i;
		for (i=0; i<event.length; i++)
		{
			this._linkedObj[event[i]][name] = 1;
			delete this._linkedObj[event[i]][name];
		}
	}

	/**
	 * Method: startup
	 *
	 *	Performs a startup call. This is generally used to get static data, configuration
	 *	variables and all sort of data that may be used by the owner of the DataSource.
	 *
	 * Parameters:
	 *
	 *	data - Aditional Information to be sent to the Server Startup Method
	 *
	 */
	p.startup = function (data)
	{
		if (!data) data = {};
		
		/* Go direct to server */
		if (!this.useSync)
		{
			data = this.protocol.encode(this.serverStartupMethod, data);
			this.connector.newRequest('thyDataSource.'+this.name+'.startup', this.serverURL, data, this._startupHandler());
			return;
		}

		/* Use synchronizer */
		thySync.request(this.name+'.startup', this.serverStartupMethod, data, 'startup', this);
	}
	
	/**
	 * Method: read
	 *
	 *	Method that connects to the server, gets the data and populate previous
	 *	linked elements
	 *
	 * Parameters:
	 *
	 *	data - Aditional Information to be sent to the Server Read Method
	 *
	 */
	p.read = function(data)
	{
		if (!data) data = {};

		/* Go direct to server */
		if (!this.useSync)
		{
			data = this.protocol.encode(this.serverReadMethod, data);
			this.connector.newRequest('thyDataSource.'+this.name+'.read', this.serverURL, data, this._readHandler());
			return;
		}

		/* Use synchronizer */
		thySync.request(this.name+'.read', this.serverReadMethod, data, 'read', this);
	}

	/**
	 * Method: write
	 *
	 *	Method that sweepOut all linked elements data, connects to the server and sends the 
	 *	data to the server
	 *
	 * Parameters:
	 *
	 *	data - Aditional Information to be sent to the Server Write Method
	 *
	 */
	p.write = function(data)
	{
		var i,sweepedData = {};
		for (i in this._linkedObj['write'])
		{
			sweepedData[i] = this._linkedObj['write'][i].sweepOut();
		}

		for (i in data)
		{
			sweepedData[i] = data[i];
		}

		/* Go direct to server */
		if (!this.useSync)
		{
			var post = this.protocol.encode(this.serverWriteMethod, sweepedData);
			this.connector.newRequest('thyDataSource.'+this.name+'.write', this.serverURL, post, this._writeHandler())
			return;
		}

		/* Use synchronizer */
		thySync.request(this.name+'.write', this.serverWriteMethod, sweepedData, 'write', this);
	}
	
	/**
	 * Method: remove
	 *
	 *	Method that connects to the server and execute remove method
	 *
	 * Parameters:
	 *
	 *	data - Aditional Information to be sent to the Server Remove Method
	 *
	 */
	p.remove = function(data)
	{
		if (!data) data = {};

		/* Go direct to server */
		if (!this.useSync)
		{
			data = this.protocol.encode(this.serverRemoveMethod, data);
			this.connector.newRequest('thyDataSource.'+this.name+'.remove', this.serverURL, data, this._removeHandler());
			return;
		}

		/* Use synchronizer */
		thySync.request(this.name+'.remove', this.serverRemoveMethod, data, 'remove', this);
	}

	/*************************************************************************\
	 *                          Private Methods                              *
	\*************************************************************************/

	/**
	 * Method: _readHandler
	 *
	 * Handler for incomming connections called from <read> method
	 *
	 * Scope:
	 *
	 *	Protected
	 *
	 */
	p._readHandler = function ()
	{
		var self = this;

		return function(response)
		{
			var data = self.protocol.decode(response);

			if (data['msg'])
			{
				showMessage(data['msg']);
			}

			if (data['status'] != 'ok')
			{
				//TODO: ERROR Handling
				return;
			}

			var i;
			data = data['data'];

			self.populate(data, 'read');
		}
	}

	/**
	 * Method: _writeHandler
	 *
	 * Handler for incomming connections called from <write> method
	 *
	 * Scope:
	 *
	 *	Protected
	 *
	 */
	p._writeHandler = function ()
	{
		var self = this;

		return function(response)
		{
			var data = self.protocol.decode(response);

			if (data['msg'])
			{
				showMessage(data['msg']);
			}

			if (data['status'] != 'ok')
			{
				//TODO: ERROR Handling
				return;
			}

			self.populate(data.data, 'write');
		}
	}

	/**
	 * Method: _removeHandler
	 *
	 * Handler for incomming connections called from <remove> method
	 *
	 * Scope:
	 *
	 *	Protected
	 *
	 */
	p._removeHandler = function ()
	{
		var self = this;

		return function(response)
		{
			var data = self.protocol.decode(response);

			if (data['msg'])
			{
				showMessage(data['msg']);
			}

			if (data['status'] != 'ok')
			{
				//TODO: ERROR Handling
				return;
			}

			self.populate(data.data, 'remove');
		}
	}

	/**
	 * Method: _startupHandler
	 *
	 * Handler for incomming connections called from <startup> method
	 *
	 * Scope:
	 *
	 *	Protected
	 *
	 */
	p._startupHandler = function ()
	{
		var self = this;

		return function(response)
		{
			var data = self.protocol.decode(response);

			if (data['msg'])
			{
				showMessage(data['msg']);
			}

			if (data['status'] != 'ok')
			{
				//TODO: ERROR Handling
				return;
			}

			var i;
			data = data['data'];

			if (data['thyConnector_messages'])
			{
				var msgs = data['thyConnector_messages'];
				
				if (msgs['default'])
				{
					if (!msgs['read']) self.setReadMessages(msgs['default']);
					if (!msgs['write']) self.setWriteMessages(msgs['default']);
					if (!msgs['remove']) self.setRemoveMessages(msgs['default']);
				}
				
				if (msgs['read']) self.setReadMessages(msgs['read']);
				if (msgs['write']) self.setWriteMessages(msgs['write']);
				if (msgs['remove']) self.setRemoveMessages(msgs['remove']);
				
				delete data['thyConnector_messages'];
			}
			
			self.populate(data, 'startup');
		}
	}
