 /*****************************************************************************\
 * ThyAPI - Thyamad Javascript API - Synchronizeer Element                    *
 * http://www.thyamad.com                                                     *
 *                                                                            *
 * Copyright (C) 2005 - Raphael Derosso Pereira                               *
 * Based on DynAPI v3.0b1                                                     *
 * -------------------------------------------------------------------------  *
 *  This library is free software; you can redistribute it and/or modify it   *
 *  under the terms of the GNU Lesser General Public License as published     *
 *  by the Free Software Foundation; either version 2.1 of the License,       *
 *  or any later version.                                                     *
 *  This library is distributed in the hope that it will be useful, but       *
 *  WITHOUT ANY WARRANTY; without even the implied warranty of                *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
 *  See the GNU Lesser General Public License for more details.               *
 \****************************************************************************/

	/**
	 * Class: thySynchronizer
	 *
	 *	This class defines a Singleton that will be responsible to accumulate
	 *	multiples calls to server, execute them and upon receival of server
	 *	response, respass them to the appropriate <thyDataSource> component
	 *
	 * Events:
	 *
	 */
	function thySynchronizer()
	{
		this.thyBase = thyBase;
		this.thyBase();
		this.DynElement = DynElement;
		this.DynElement();

		this.requested = false;
		this.syncID;
		this.serverURL;
		this.syncMethod;
		this.inProgress = false;

		// thyConnector is a Singleton!
		if (!window.thyConnectorObj)
		{
			window.thyConnectorObj = new thyConnector();
			window.thyConnectorObj.setVisible(true);
		}
			
		this.connector = thyConnectorObj;

		//** Initialize synchronization call **\\
		var self = this;
		dynapi.onLoad(function(){self.setInterval(100)});
	}

	p = dynapi.setPrototype('thySynchronizer',['thyBase','DynElement']);

	/**
	 * Method: setProgressHolder
	 *
	 *	Tells the Connector to put its message on the specified Progress Holder
	 *
	 * Parameters:
	 *
	 *	holder - The DynLayer (or derivate) that will hold the Messages
	 *
	 */
	p.setProgressHolder = function(holder)
	{
		this.connector.setProgressHolder('thySynchronizer', holder);
	}

	/**
	 * Method: setMessages
	 *
	 *	Tells the Connector which messages are to be shown when calling synchronize
	 *
	 * Parameters:
	 *
	 *	messages - Array with messages. There are 3 possible messages:
	 *
	 *		- messages[0] - Contacting Server
	 *		- messages[1] - Server Contacted
	 *		- messages[2] - Loading Server Data
	 *
	 */
	p.setReadMessages = function (messages)
	{
		if (typeof(messages) != 'object') throw({result: 'Invalid Message', location: 'thyDataSource.setReadMessages'});

		this.connector.setProgressContent('thySynchronizer', 1, messages[0]);
		this.connector.setProgressContent('thySynchronizer', 2, messages[1]);
		this.connector.setProgressContent('thySynchronizer', 3, messages[2]);
	}

	/**
	 * Method: setProtocol
	 *
	 *	Sets the protocol to be used by Synchronizer
	 *
	 * Parameters:
	 *
	 *	protocol - The Protocol Object aready instantiated
	 */
	p.setProtocol = function (protocol)
	{
		this.protocol = protocol;
		this.serverURL = protocol.getServerURL();
	}
	
	/**
	 * Method: setSyncMethod
	 *
	 *	Sets up the method that will handle synchronization calls
	 *
	 * Parameter:
	 *
	 *	methodName - The complete name of the method in a server understandable
	 *	             way.
	 */
	p.setSyncMethod = function (methodName)
	{
		this.syncMethod = methodName;
	}

	/**
	 * Method: setInterval
	 *
	 *	Sets internal synchronize check interval. Defaults to 100ms
	 *
	 * Parameter:
	 *
	 *	interval - The number of miliseconds to wait between sync verifications
	 *
	 */
	p.setInterval = function (interval)
	{
		if (typeof(interval) != 'number') return;
		
		this.interval = interval;
		if (this.syncID)
		{
			this.clearInterval(this.syncID);
		}
		
		var self = this;
		this.syncID = setInterval(function(){self.synchronize()}, this.interval);
	}

	/**
	 * Method: request
	 *
	 *	Performs a request to <thySynchronizer>. This should be called from
	 *	<thyDataSource> when any user action that demands server interaction
	 *	is performed.
	 *
	 * Parameters:
	 *
	 *	name   - The name of the object plus an unique identifier, so if same
	 *	         object performs 2 calls to server, both will be executed.
	 *
	 *	method - The method that this object need to be called.
	 *
	 *	params - Method parameters.
	 *
	 *	event  - The event name (startup, read or write)
	 *
	 *	obj    - The object itself so server response can be populated to it
	 *
	 */
	p.request = function (name, method, params, event, obj)
	{
		if (!this.requested)
		{
			this.requested = {};
		}
		
		this.requested[name] = {method: method, params: params, event: event, obj: obj};
	}
	
	/**
	 * Method: synchronize
	 *
	 *	This method is automatically called every pre-defined period of time.
	 *	It checks for client requests and performs the appropriate action,
	 *	like a sweepOut() call to each <thyDataSource> which has reported an
	 *	action request. After that, it will communicate with server and wait
	 *	its response.
	 *
	 */
	p.synchronize = function ()
	{
		if (!this.requested || this.inProgress) return;

		this.inProgress = true;
		var i, data = {};
		for (i in this.requested)
		{
			data[i] = {method: this.requested[i].method, params: this.requested[i].params};
		}

		var post = this.protocol.encode(this.syncMethod, data);
		this.connector.setContentType(this.protocol.getContentType());
		this.connector.newRequest('thySynchronizer', this.serverURL, post, this._syncHandler())
	}

	/*************************************************************************\
	 *                       Group: Private Methods                          *
	\*************************************************************************/

	/**
	 * Method: _syncHandler
	 *
	 *	Method that handles responses from server. It verifies integrity of sync
	 *	data and pass responses to appropriate <thyDataSources>
	 *
	 */
	p._syncHandler = function ()
	{
		var self = this;
		return function (response)
		{
			try
			{
				var data = self.protocol.decode(response);
				self.inProgress = false;

				if (data['msg'])
				{
					showMessage(data['msg']);
				}
				
				if (data['status'] != 'ok')
				{
					self.requested = null;
					return;
				}

				data = data.data;

				var i;
				for (i in data)
				{
					if (!self.requested ||
						!self.requested[i] || 
						!self.requested[i].obj || 
						!self.requested[i].obj.populate)
					{
						continue;
					}

					if (data[i]['msg'])
					{
						showMessage(data[i]['msg']);
					}

					// Check integrity
					if (data[i]['status'] != 'ok')
					{
						continue;
					}
					
					self.requested[i].obj.populate(data[i]['data'], self.requested[i]['event']);
				}
				self.requested = null;
			}
			catch(e)
			{
				self.requested = null;
				self.inProgress = false;
				throw(e);
			}
		}
	}

	/*************************************************************************\
	 *                       Group: Initialization                           *
	\*************************************************************************/

	/**
	 * Initialize thySynchronizer, as it is a Singleton
	 *
	 *	Use thySynchronizer as the object
	 */

	var thySync = new thySynchronizer();
