 /***************************************************************************\
 * ThyAPI - Thyamad Javascript API - Button Element                          *
 * http://www.thyamad.com                                                    *
 *                                                                           *
 * Copyright (C) 2005 - Raphael Derosso Pereira                              *
 * Based on DynAPI v3.0b1                                                    *
 * ------------------------------------------------------------------------- *
 *  This library is free software; you can redistribute it and/or modify it  *
 *  under the terms of the GNU Lesser General Public License as published    *
 *  by the Free Software Foundation; either version 2.1 of the License,      *
 *  or any later version.                                                    *
 *  This library is distributed in the hope that it will be useful, but      *
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *  See the GNU Lesser General Public License for more details.              *
 \***************************************************************************/

	/**
	 * Class: thyButton
	 *
	 *	Button Widget
	 *
	 *	The button can be toggle or normal one. The available component user 
	 *	events are onbuttondown, onbuttonup for toggle button and onclick for
	 *	non-toggle button.
	 *
	 * CSS classes:
	 *
	 *	- .thyButton,
	 *	- .thyButton_pushed,
	 *
	 *	- .<object name>,
	 *	- .<object name>_pushed,
	 *
	 * Events:
	 *
	 *	- onclick
	 *	- onbuttondown
	 *	- onbuttonup
	 *
	 * Author:
	 *
	 *	Raphael Derosso Pereira
	 *
	 *
	 */
	function thyButton(name,caption,toggle)
	{
		this.thyPanel = thyPanel;
		this.thyPanel(name,caption);
		
		// Attributes
		this.contentRelative = false;
		this.toggle = toggle || false;
		this.toggleStatus = 0;
		this.count = 0;
		this.addCSSClass('thyButton');

		this.setTextSelectable(false);
		this.setCursor('hand');

		//if (!this.thyButton) this.onPreCreate(this.initThyButton);
		this.onPreCreate(this.initThyButton);
	}
	
	p = dynapi.setPrototype('thyButton','thyPanel');

	/**
	 * Method: initThyButton
	 *
	 *	Calls all required initialization procedures. This will be called just
	 *	before creating the object.
	 *
	 */
	p.initThyButton = function ()
	{
		this.addEventListener({
			onmousedown: this._getOnButtonDown(),
			onmouseup  : this._getOnButtonUp(),
			onmouseover: this._getOnMouseOver(),
			onmouseout : this._getOnMouseOut()
		});
		
		dynapi.document.addEventListener({
			onmouseup: this._getDocMouseUp()
		});
	}

	/**
	 * Method: populate
	 *
	 *	Update the button caption
	 *
	 */
	p.populate = function(data)
	{
		if (typeof(data) != 'string' && typeof(data) != 'number') throw({result: 'Invalid Caption', location: 'thyButton.populate'});
		this.setHTML(data);
	}

	/**
	 * Method: sweepOut
	 *
	 *	Returns the current caption of this button
	 *
	 */
	p.sweepOut = function()
	{
		return this.getHTML();
	}

	/**
	 * Method: setToggle
	 *
	 *	Sets the type of button to toggle or to conventional
	 *
	 * Parameters:
	 *
	 *	bool - True -> toggle button, false -> normal button
	 *
	 */
	p.setToggle = function(bool)
	{
		this.toggle = (bool) ? true : false;
	}

	/**
	 * Method: pressButton
	 *
	 *	Has the same effect as pushing the button with the mouse
	 *
	 * Parameter:
	 *
	 *	event - (boolean) Call event?
	 */
	p.pressButton = function (event)
	{
		if (event == null) event = true;
		this.cssClasses['default'] = ['default','pushed'];
		this.setCSSClass();
		if (this.toggle && this.toggleStatus == 1) this.toggleStatus = 2;
		else if (this.toggle && this.toggleStatus == 0)
		{
			this.toggleStatus = 1;
			if (event) this.invokeEvent('buttondown');
		}
		else if (!this.toggle) this.toggleStatus = 1;
	}

	/**
	 * Method: releaseButton
	 *
	 *	Has the same effect as releasing the button with the mouse
	 *
	 * Parameter:
	 *
	 *	event - (boolean) Call event?
	 */
	p.releaseButton = function (event)
	{
		if (this.toggle && this.toggleStatus == 0) return;
			
		if (event == null) event = true;
		
		this.cssClasses['default'] = ['default'];
		this.toggleStatus = 0;
		this.setCSSClass();
		this.onmouseout();
		
		if (event) this.invokeEvent('buttonup');
	}
	
	/*************************************************************************\
	 *                          Private Methods                              *
	\*************************************************************************/

	/**
	 * Method: _cleanUp
	 *
	 *	Overwritten method that puts button back to it initial state
	 *
	 */
	p._cleanUp = function ()
	{
		this.releaseButton();
	}

	/**
	 * Method: _reset
	 *
	 *	Overwritten method that puts button back to it initial state
	 *
	 */
	p._reset = p._cleanUp;
	
	/**
	 * Method: _getOnButtonDown
	 *
	 *	This invokes the function associated with the onbuttondown event and
	 *	garantees that the button has the right CSS class
	 *
	 */
	p._getOnButtonDown = function()
	{
		var self = this;
		return function (e)
		{
			self.pressButton();
		}
	}
	
	/**
	 * Method: _getOnButtonUp
	 *
	 *	This invokes the function associated with the onbuttonup event and
	 *	garantees that the button has the right CSS class
	 *
	 */
	p._getOnButtonUp = function()
	{
		var self = this;
		return function (e)
		{
			if ((!self.toggle && self.toggleStatus == 1) || (self.toggle && self.toggleStatus == 2))
			{
				self.releaseButton();
			}
		}
	}
	
	/**
	 * Method: _getOnMouseOver
	 *
	 *	Returns a function that will be called when 'onmouseover' event
	 *	occurs.
	 *
	 *	This function specifically handles the insertion of the pushed
	 *	CSS class when the button has been pushed down and the mouse has
	 *	gone out without the user releasing the mouse button. If the user
	 *	gets back with the mouse holding pressed over the button, the
	 *	button must appear as pushed.
	 *
	 */
	p._getOnMouseOver = function ()
	{
		var self = this;
		return function (e)
		{
			if (self.toggle) return;
			if (self.toggleStatus == 1) self.cssClasses['default'] = ['default', 'pushed'];
		}
	}
	
	/**
	 * Method: _getOnMouseOut
	 *
	 *	Returns a function that will be called when 'onmouseout' event
	 *	occurs.
	 *
	 *	This function specifically handles the removal of the pushed CSS 
	 *	class when the button is not a toggle button
	 *
	 */
	p._getOnMouseOut = function ()
	{
		var self = this;
		return function (e)
		{
			if (self.toggle) return;
			self.cssClasses['default'] = ['default'];
			self.setCSSClass();
		}
	}

	/**
	 * Method: _getDocMouseUp
	 *
	 *	Checks on every mouseup event if the event was done out of the button
	 *	and in that case, changes the toggle status of the button to 0
	 *
	 */
	p._getDocMouseUp = function ()
	{
		var self = this;
		return function (e)
		{
			if (self != e.getOrigin() && !self.toggle) self.toggleStatus = 0;
		}
	}
