 /***************************************************************************\
 * ThyAPI - Thyamad Javascript API - CheckBox Element                        *
 * http://www.thyamad.com                                                    *
 *                                                                           *
 * Copyright (C) 2005 - Raphael Derosso Pereira                              *
 * Based on DynAPI v3.0b1                                                    *
 * ------------------------------------------------------------------------- *
 *  This library is free software; you can redistribute it and/or modify it  *
 *  under the terms of the GNU Lesser General Public License as published    *
 *  by the Free Software Foundation; either version 2.1 of the License,      *
 *  or any later version.                                                    *
 *  This library is distributed in the hope that it will be useful, but      *
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *  See the GNU Lesser General Public License for more details.              *
 \***************************************************************************/

	/**
	 * Class: thyCheckBox  
	 *
	 *	This is a checkbox element.
	 *
	 * CSS Classes:
	 *
	 *	- .thyCheckBox
	 *	- .thyCheckBox_checked
	 *	- .thyCheckBox_label
	 *	- .thyCheckBox_contents
	 *	- .thyCheckBox_contents_checked
	 *	- .thyLabelPanel_label
	 *	- .thyLabelPanel_contents
	 *
	 * Events:
	 *
	 *	onclick   - When checkbox has been clicked
	 *	onchange  - When checkbox has been changed
	 *	oncheck   - When checkbox has been checked
	 *	onuncheck - When checkbox has been unchecked
	 *
	 */
	function thyCheckBox(name)
	{
		this.thyLabelPanel = thyLabelPanel;
		this.thyLabelPanel(name);

		//** Attributes **\\
		this.checkedState = false;

	/*	this.cssClasses['checked'] = ['default','checked'];
		this.label.cssClasses = this.cssClasses;
		this.contents.cssClasses = this.cssClasses;*/
		this.label.cssExtraClasses = this.cssExtraClasses;
		this.contents.cssExtraClasses = this.cssExtraClasses;

		this.addCSSClass('thyCheckBox');

		this.contents.setRelativeness(false);
		
		//** Events **\\
		this.addEventListener({onclick: this._eventOnClick()});
	}

	p = dynapi.setPrototype('thyCheckBox','thyLabelPanel');

	/**
	 * Method: check
	 *
	 *	Sets the element as checked
	 *
	 */
	p.check = function ()
	{
		var oldState = this.checkedState;
		
		this.checkedState = true;
		this.cssExtraClasses['checked'] = {};
		this.cssExtraClasses['checked'][this.cssActualSelection] = ['checked'];
		this.setCSSClass(this.cssActualSelection);
		//this.setCSSClass('checked');

		if (this.checkedState != oldState)
		{
			this.invokeEvent('change', new DynEvent('change', this));
			this.invokeEvent('check', new DynEvent('check', this));
		}
	}

	/**
	 * Method: uncheck
	 *
	 *	Sets the element as unchecked
	 *
	 */
	p.uncheck = function ()
	{
		var oldState = this.checkedState;
		
		this.checkedState = false;
		delete this.cssExtraClasses['checked'];
		this.setCSSClass(this.cssActualSelection);

		if (this.checkedState != oldState)
		{
			this.invokeEvent('change', new DynEvent('change', this));
			this.invokeEvent('uncheck', new DynEvent('uncheck', this));
		}
	}

	/**
	 * Method: switchCheckedState
	 *
	 *	Switch the actual checked state
	 *
	 */
	p.switchCheckedState = function ()
	{
		if (this.checkedState) this.uncheck();
		else this.check();
	}

	p._thyLabelPaneladdEventListener = p.addEventListener;
	/**
	 * Method: addEventListener
	 *
	 *	Appends the event listener to the contents element, as this is the
	 *	correct place for this widget
	 *
	 */
	p.addEventListener = function (event)
	{
		this.contents.addEventListener(event);
	}

	/*************************************************************************\
	 *                        Group: Private Methods                         *
	\*************************************************************************/
	
	/**
	 * Method: _populateContents
	 *
	 *	Sets the CheckBox as checked or unchecked
	 *
	 * Parameter:
	 *
	 *	checkedState - (boolean) true/false
	 *
	 */
	p._populateContents = function (checkedState)
	{
		if (checkedState) this.check();
		else this.uncheck();
	}

	/**
	 * Method: _sweepOutContents
	 *
	 *	Returns the actual checked state
	 *
	 */
	p._sweepOutContents = function ()
	{
		return this.checkedState;
	}

	/**
	 * Method: _reset
	 *
	 *	Resets the checkbox to unchecked state
	 *
	 */
	p._reset = function ()
	{
		this.uncheck();
	}

	/**
	 * Method: _cleanUp
	 *
	 *	Wrapper to reset
	 *
	 */
	p._cleanUp = p._reset;
	
	/*************************************************************************\
	 *                          Group: Events                                *
	\*************************************************************************/

	/**
	 * Method: _eventOnClick
	 *
	 *	Sets the default behaviour when onclick events
	 *
	 */
	p._eventOnClick = function ()
	{
		var self = this;
		return function (e)
		{
			self.switchCheckedState();
		}
	}
