 /***************************************************************************\
 * ThyAPI - Thyamad Javascript API - Drop-Down Combo Box Element             *
 * http://www.thyamad.com                                                    *
 *                                                                           *
 * Copyright (C) 2005 - Raphael Derosso Pereira                              *
 * Based on DynAPI v3.0b1                                                    *
 * ------------------------------------------------------------------------- *
 *  This library is free software; you can redistribute it and/or modify it  *
 *  under the terms of the GNU Lesser General Public License as published    *
 *  by the Free Software Foundation; either version 2.1 of the License,      *
 *  or any later version.                                                    *
 *  This library is distributed in the hope that it will be useful, but      *
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *  See the GNU Lesser General Public License for more details.              *
 \***************************************************************************/


	/**
	 * Class: thyDropDownBox
	 *
	 *	This is the Drop Down Combo Box Element. It provides a <thyGridCell>,
	 *	a <thyButton> and a <thyListBox> working together to provide a
	 *	Drop-Down Combo Box Widget.
	 *
	 * CSS Classes:
	 *
	 *	- <thyPanel> classes
	 *	- <thyLabelPanel> classes
	 *	- <thyEditBox> classes
	 *
	 *	.thyDropDownBox                  - The class of main element, derived from <thyEditBox>
	 *	.thyDropDownBox_button           - The <thyButton> element class
	 *	.thyDropDownBox_listBox          - The <thyListBox> element class
	 *
	 *	.<object name>                   - The class of main element, derived from <thyEditBox>
	 *	.<object name>_edit              - The <thyEditBox> element class
	 *	.<object name>_button            - The <thyButton> element class
	 *	.<object name>_listBox           - The <thyListBox> element class
	 *
	 * Events:
	 *
	 *	onselect - When user selects an option
	 *	onchange - When there's any change in value (by reset and populate considered)
	 *
	 * See Also:
	 *
	 *	- <thyEditBox>
	 *	- <thyButton>
	 *	- <thyListBox>
	 *	- <thyLabelPanel>
	 *	- <thyPanel>
	 *
	 */
	function thyDropDownBox(name,label)
	{
		this.thyLabelPanel = thyLabelPanel;
		this.thyLabelPanel(name,label);

		// Atributes
		this.dropEdit   = new thyEditBox(this.name+'_edit');
		this.dropButton = new thyButton(this.name+'_button');
		this.dropList   = new thyListBox(this.name+'_listBox');

		this.dropSelected = 0;
		this.dropDefault  = 0;

		// Initialization \\
		this.dropButton.setZIndex({above: this.contents});
		this.dropButton.setToggle(true);

		//this.dropList.setY(0);
		this.dropList.setDisplay(false);

		// Events \\
		this.dropList.addEventListener({
			onselectrow: this._eventSelectOption(),
			onunselectrow: this._eventSelectOption()
		});
		this.dropButton.addEventListener({
			onbuttondown: this._eventShowListBox(),
			onbuttonup: this._eventHideListBox()
		});

		//FIXME: This is not implemented yet.
		//this.dropEdit.setReadOnly(true);
		/*this.dropEdit.addEventListener({
			onkeypress: this._eventSearchRow(),
			onblur: this._eventSelectOptionByString(),
			onfocus: this._eventShowListBoxOnFocus()
		});*/
		
		this.addCSSClass('thyDropDownBox');

		this._thyLabelPanelAddChild(this.dropEdit);
		this._thyLabelPanelAddChild(this.dropButton);
		//this._thyLabelPanelAddChild(this.dropList);

		if (!this.thyDropDownBox) this.onPreCreate(this.initThyDropDownBox);
	}

	p = dynapi.setPrototype('thyDropDownBox', 'thyLabelPanel');

	/**
	 * Method: initThyDropDownBox
	 *
	 *	Initialization method called before creation
	 *
	 */
	p.initThyDropDownBox = function ()
	{
		dynapi.document.addChild(this.dropList);
	}
	
	/**
	 * Method: setDefaultValue
	 *
	 *	Sets the default value when widget is reset
	 *
	 * Parameters:
	 *
	 *	value - The default value
	 *
	 */
	p.setDefaultValue = function (value)
	{
		this.dropDefault = value;
	}
	
	/**
	 * Method: getSelectedIndex
	 *
	 *	Returns the index of the selected option
	 *
	 */
	p.getSelectedIndex = function ()
	{
		return this.dropSelected;
	}

	/**
	 * Method: getSelectedValue
	 *
	 *	Returns the value of the selected option
	 *
	 */
	p.getSelectedValue = function ()
	{
		return this.dropList.getCell(this.dropSelected, 0);
	}
	
	/**
	 * Method: getSelectedContent
	 *
	 *	Returns the content of the selected option
	 *
	 */
	p.getSelectedContent = function ()
	{
		return this.dropList.getCell(this.dropSelected, 1);
	}
	
	/**
	 * Method: getSelected
	 *
	 *	Returns a <thyCollection> with the index, the value and the content 
	 *	of the selected option
	 *
	 */
	p.getSelected = function ()
	{
		if (this.dropList.getRowsCount())
		{
			var sel = this.dropList.getRow(this.dropSelected);
			sel.changeElementName({elm: sel.getElementByIndex(0), name: 'value'});
			sel.changeElementName({elm: sel.getElementByIndex(1), name: 'content'});
			sel.add(this.dropSelected, 'index');
		}
		else
		{
			sel = new thyCollection();
		}
		return sel;
	}

	/**
	 * Method: getContentByValue
	 *
	 *	Returns the associated content with the specified value
	 *
	 * Parameter:
	 *
	 *	value - The value associated with the needed content
	 *
	 */
	p.getContentByValue = function (value)
	{
		return this.dropList.getContentByValue(value);
	}
	
	p._thyLabelPanelAddChild = p.addChild;
	/**
	 * Method: addChild
	 *
	 *	Overloaded method that prevents the insertion of children in <thyDropDownBox>
	 *
	 */
	p.addChild = function ()
	{
		throw({result: 'No meaning on adding child to thyDropDownBox', location:'thyDropDownBox.addChild'});
	}

	p._thyLabelPanelRemoveChild = p.removeChild;
	/**
	 * Method: removeChild
	 *
	 *	Overloaded method that prevents the removal of children of <thyDropDownBox>
	 *
	 */
	p.removeChild = function ()
	{
		throw({result: 'No meaning on adding child to thyDropDownBox', location:'thyDropDownBox.addChild'});
	}
	
	p._thyLabelPanelSetName = p.setName;
	/**
	 * Method: setName
	 *
	 *	Overwritten method that sets the name of main, button and listbox
	 *	and refreshes CSS Classes
	 *
	 * Parameter:
	 *
	 *	name - The new widget name
	 *
	 */
	p.setName = function (name)
	{
		if (name == null || this.name == name) return;
		
		this._thyLabelPanelSetName(name);
		if (this.dropButton)
		{
			this.dropEdit.setName(name+'_edit');
			this.dropButton.setName(name+'_button');
			this.dropList.setName(name+'_listBox');
		}
	}
	
	p._thyLabelPanelSetCSSClass = p.setCSSClass;
	/**
	 * Method: setCSSClass
	 *
	 *	Overloaded method that sets the CSS Class of main, button and listbox
	 *
	 * Parameter:
	 *
	 *	type - The CSS type
	 *
	 */
	p.setCSSClass = function (type)
	{
		this._thyLabelPanelSetCSSClass(type);
		if (this.dropButton)
		{
			this.dropEdit.setCSSClass(type);
			this.dropButton.setCSSClass(type);
			this.dropList.setCSSClass(type);
		}
	}
	
	/*************************************************************************\
	 *                        Group: Private Methods                         *
	\*************************************************************************/
	
	p._thyLabelPanel_destroy = p._destroy;
	/**
	 * Method: _destroy
	 *
	 *	Overloaded method that removes dropList from memory
	 *
	 */
	p._destroy = function ()
	{
		this.dropList._destroy();
		this._thyLabelPanel_destroy();
	}
	
	/**
	 * Method: _cleanUp
	 *
	 *	Removes all rows from dropList and clear dropEdit
	 *
	 */
//	p._cleanUp = function ()
//	{
//	}

	/**
	 * Method: _reset
	 *
	 *	Resets dropList to defaultValue
	 *
	 */
	p._reset = function ()
	{
		this._populateContents(this.dropDefault);
	}
	
	/**
	 * Method: _populateContents
	 *
	 *	Sets the contents of the <thyDropDownBox>. This is the same populate as
	 *	<thyListBox>::populate, so pass correct formated data.
	 *
	 * Parameter:
	 *
	 *	data - The <thyDropDownBox> possible values
	 *
	 */
	p._populateContents = function (data)
	{
		var oldSelected = this.dropSelected;
		
		if (typeof(data) == 'number')
		{
			data = {index: data};
		}
		
		if (typeof(data) == 'object')
		{
			var index = data.index;
			var value = data.value;
			
			if (data[0])
			{
				delete data.index;
				delete data.value;
				
				var contents = [];
				for (i in data)
				{
					contents.push(data[i]);
				}
				
				this.dropList.populate({contents: contents});
				if (index == null && value == null)
				{
					this.reset();
					return;
				}
			}
			
			if (index != null)
			{
				var row = this.dropList.getRowElement(index);
				if (!row) return;
				this.dropEdit.populate(row.getCell(1));
				this.dropSelected = index;
			}
			else if (value != null)
			{
				var i,nRows = this.dropList.getRowsCount();
				
				for (i=0; i<nRows; i++)
				{
					if (this.dropList.getRowElement(i).getCellElement(0).sweepOut() == value)
					{
						this.dropEdit.populate(this.dropList.getRowElement(i).getCellElement(1).sweepOut());
						this.dropSelected = i;
						break;
					}
				}
			}
		}
		
		if (this.dropSelected != oldSelected)
		{
			this.invokeEvent('change', new DynEvent('change', this));
		}
	}

	/**
	 * Method: _sweepOutContents
	 *
	 *	Gets the value that was selected by the user.
	 *
	 */
	p._sweepOutContents = function ()
	{
		return this.getSelected().sweepOut();
	}

	p._thyLabelPanel_addCSSClass = p._addCSSClass;
	/**
	 * Method: _addCSSClass
	 *
	 *	Overwritten method that adds the class to main, button and listbox
	 *
	 * Parameter:
	 *
	 *	cssClass - The cssClass to be added
	 *
	 */
	p._addCSSClass = function (cssClass)
	{
		this._thyLabelPanel_addCSSClass(cssClass);
		if (this.dropButton)
		{
			this.dropEdit._addCSSClass(cssClass+'_edit');
			this.dropButton._addCSSClass(cssClass+'_button');
			this.dropList._addCSSClass(cssClass+'_listBox');
		}
	}
	
	p._thyLabelPanel_removeCSSClass = p._removeCSSClass;
	/**
	 * Method: _removeCSSClass
	 *
	 *	Overwritten method that removes the specified CSS Class from main,
	 *	button and listbox
	 *
	 * Parameter:
	 *
	 *	cssClass - The cssClass to be removed
	 *
	 */
	p._removeCSSClass = function (cssClass)
	{
		this._thyLabelPanel_removeCSSClass(cssClass);
		if (this.dropButton)
		{
			this.dropEdit._removeCSSClass(cssClass+'_edit');
			this.dropButton._removeCSSClass(cssClass+'_button');
			this.dropList._removeCSSClass(cssClass+'_list');
		}
	}

	/**
	 * Method: _hideListBox
	 *
	 *	Hides the <thyListBox> element
	 *
	 */
	p._hideListBox = function ()
	{
		//this.dropList.setZIndex(this.oldZIndex);
		this.dropList.setDisplay(false);
		this.dropButton.releaseButton(false);
		if (!dynapi.document._opennedDropDownBoxes) return;
		delete dynapi.document._opennedDropDownBoxes[this.name];
	}

	/**
	 * Method: _showListBox
	 *
	 *	Shows the <thyListBox>element
	 *
	 */
	p._showListBox = function ()
	{
		// Step 1: Select actual selected row \\
		var row = this.dropList.getRowElement(this.dropSelected);
		row.selectOnly();

		// Step 2: Display Box \\
		/*this.oldZIndex = this.dropList.getZIndex();
		this.dropList.setZIndex({topmost: true});
		this.dropList.setDisplay(true);

		// Step 3: Position the Box correctly \\
		var dropListH = this.dropList.getHeight();
		this.dropList.setY(this.dropEdit.getHeight()/2-dropListH/2);
		
		var dropListY = this.dropList.getAbsoluteY();
		if (dropListY < 0)
		{
			dropListY = this.dropList.getY() - dropListY;
			this.dropList.setY(dropListY);
		}
		
		this.dropList.setDisplay(false);
		this.dropList.setDisplay(true);
		return;*/

		// Step 2: Adjust List position and size and show it \\
		var dropEditX = this.dropEdit.getAbsoluteX();
		var dropEditW = this.dropEdit.getWidth();
		
		if (!this.dropListAdjusted)
		{
			this.dropList.setMinWidth(dropEditW+30);
			this.dropListAdjusted = true;
		}
		
		this.dropList.setX(dropEditX);
		this.dropList.setDisplay(true);

		this.dropList.setZIndex({topmost: true});
		
		var listBoxW = this.dropList.getWidth();
		var diff = listBoxW < dropEditW ? 0 : listBoxW - dropEditW;
		
		this.dropList.setX(dropEditX-diff);
		this.dropList.setY(this.dropEdit.getAbsoluteY()+this.dropEdit.getHeight()/2-this.dropList.getHeight()/2);
		
		var dropListY = this.dropList.getAbsoluteY();
		if (dropListY < 0)
		{
			dropListY = this.dropList.getY() - dropListY;
			this.dropList.setY(dropListY);
		}
		
		var dropListX = this.dropList.getAbsoluteX();
		if (dropListX < 0)
		{
			dropListX = this.dropList.getX() - dropListX;
			this.dropList.setX(dropListX);
		}

		// Step 3: Put openned drop down in document list so it will be closed
		//	when an onclick event happens
		if (!dynapi.document._opennedDropDownBoxes)
		{
			dynapi.document._opennedDropDownBoxes = {};
		}
		
		var self = this;
		setTimeout(function(){dynapi.document._opennedDropDownBoxes[self.name] = self;},100);
	}

	/*************************************************************************\
	 *                        Group: Events Methods                          *
	\*************************************************************************/
	

	/**
	 * Method: _eventShowListBox
	 *
	 *	Returns the function that shows the ListBox
	 *
	 */
	p._eventShowListBox = function ()
	{
		var self = this;
		return function (e)
		{
			self._showListBox();
		}
	}
	
	/**
	 * Method: _eventShowListBoxOnFocus
	 *
	 *	Returns the function that shows the ListBox and press the button
	 *
	 */
	p._eventShowListBoxOnFocus = function ()
	{
		var self = this;
		return function (e)
		{
			self._showListBox();
			self.dropButton.pressButton(false);
		}
	}
	
	p._eventHideListBox = function ()
	{
		var self = this;
		return function (e)
		{
			self._hideListBox();
		}
	}
	
	p._eventSearchRow = function ()
	{
		var self = this;
		return function (e)
		{
			var rowIdx = self.dropList.search(self.dropEdit.sweepOut());
			if (rowIdx == null) return;
			//self.dropList.setY(self.dropList.getY()-self.dropList.getRowElement(rowIdx).getY());
			var rowY = self.dropList.getRowElement(rowIdx).getY();
			var listY = self.dropList.getY();
			self.dropList.setY(listY-rowY);
		}
	}
	
	p._eventSelectOption = function ()
	{
		var self = this;
		return function (e)
		{
			var o = e.getOrigin();
			var oldSelected = self.dropSelected;
			var old = self.dropList.getRowElement(oldSelected);
			old.unselectOnly();
			
			self.dropSelected = o.getIndex();
			
			o.unselectOnly();
			self._hideListBox();
			//self.dropButton.releaseButton(false);
			o.getCellElement(1).onmouseout();

			self.invokeEvent('select', new DynEvent('select', self));

			if (oldSelected != self.dropSelected)
			{			
				self.dropEdit.populate(o.getCell(1));
				self.invokeEvent('change', new DynEvent('change', self));
			}
		}
	}
	
	p._eventSelectOptionByString = function ()
	{
		var self = this;
		return function (e)
		{
			self.dropSelected = self.dropList.search(self.dropEdit.sweepOut());
			if (self.dropSelected == null) return;
			self.dropEdit.populate(self.dropList.getCell(self.dropSelected, 1));
			self._hideListBox();
			self.dropButton.releaseButton(false);
			self.invokeEvent('select', new DynEvent('select', self));
		}
	}

	/****************************************************************************\
	 *                        Group: Misc Operations                            *
	\****************************************************************************/

	/**
	 * Function: _eventHideAllDropDown
	 *
	 *	Checks on every mousedown event if the event was done out of dropList
	 *	and in that case, just hide the list
	 *
	 */
	var eventHideAllDropDown = function (e)
	{
		var i, doc = dynapi.document;
		var orig = e.getOrigin();
		var list;
		
		if (orig.isClass('thyListBox'))
		{
			list = orig;
		}
		else if (orig.isClass('thyGridCell'))
		{
			list = orig.parent.parent.parent.parent;
		}
		
		for (i in doc._opennedDropDownBoxes)
		{
			if (list == doc._opennedDropDownBoxes[i].dropList) continue;
			doc._opennedDropDownBoxes[i]._hideListBox();
			delete doc._opennedDropDownBoxes[i];
		}
	}

	dynapi.document.addEventListener({
		onmousedown: eventHideAllDropDown
		//onclick: eventHideAllDropDown,
		//ondragstart: eventHideAllDropDown
	});
