 /***************************************************************************\
 * ThyAPI - Thyamad Javascript API - Table Grid Cell Element                 *
 * http://www.thyamad.com                                                    *
 *                                                                           *
 * Copyright (C) 2005 - Raphael Derosso Pereira                              *
 * Based on DynAPI v3.0b1                                                    *
 * ------------------------------------------------------------------------- *
 *  This library is free software; you can redistribute it and/or modify it  *
 *  under the terms of the GNU Lesser General Public License as published    *
 *  by the Free Software Foundation; either version 2.1 of the License,      *
 *  or any later version.                                                    *
 *  This library is distributed in the hope that it will be useful, but      *
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *  See the GNU Lesser General Public License for more details.              *
 \***************************************************************************/


	/**
	 * Class: thyGridCell
	 *
	 *	This is the basic Cell element. It should provides all functionalities 
	 *	that a Table Cell element should have.
	 *
	 * CSS Classes:
	 *
	 *	- .thyGridCell_edit             - when in simple edit mode
	 *	- .thyGridCell_editFull         - when in full edit mode
	 *	- .thyGridCell_selected         - when selected
	 *	- .thyGridCell_header           - when type is header
	 *	- .thyGridCell_header_selected  - when type is header and it is selected
	 *
	 *	- .<object name>_edit            - when in simple edit mode
	 *	- .<object name>_editFull        - when in full edit mode
	 *	- .<object name>_selected        - when selected
	 *	- .<object name>_header          - when type is header
	 *	- .<object name>_header_selected - when type is header and it is selected
	 *
	 * Package: ThyAPI
	 *
	 * Namespace: ThyWidgets
	 *
	 * See Also:
	 *	- <thyGrid>
	 *	- <thyPanel>
	 *
	 */
	function thyGridCell(name, html)
	{
		this.thyPanel = thyPanel;
		this.thyPanel(name,null);

		// Atributes
		this.cellMode = 0;
		this.cellType = 0;
		this.cellSelected = false;
		this.setRelativeness(false);

		this.cssClasses = {
			'default': ['default'],
			selected : ['default', 'selected'],
			header   : ['header'],
			header_selected : ['header', 'header_selected']
		};
		this.addCSSClass('thyGridCell');

		var self = this;
		this.addEventListener({ onclick: function(e){
			if (e.getOrigin() == self.cellContents) 
				self._selectEvent()
		}});
	}

	p = dynapi.setPrototype('thyGridCell', 'thyPanel');

	/**
	 * Method: populate
	 *
	 *	Sets the contents of the cell. 
	 *
	 * Parameter:
	 *
	 *	data - The contents of the Cell, that can be:
	 *
	 *	- A non-object data (will be included as HTML to the contents of the cell)
	 *	- A <thyPanel> or derivate class object that will be added as child.
	 *	- A <thyCollection> of <thyPanel> or derivate classe objects, that will
	 *	be added sequentially as children.
	 *
	 */
	p.populate = function (data)
	{
		if (typeof(data) != 'object') this.setHTML(data);
		else if (typeof(data.isFromClass) == 'function')
		{
			if(data.isFromClass('thyPanel'))
			{
				this._destroyAllChildren();
				this.addChild(data);
			}
			else if (data.isFromClass('thyCollection'))
			{
				try
				{
					this._destroyAllChildren();
					var i,nData = data.getElementsCount();
					
					for (i=0; i<nData; i++)
					{
						this.addChild(data.getElementById(i));
					}
				}
				catch(e)
				{
					return;
				}
			}
		}
		else
		{
			try
			{
				this.setHTML(data);
			}
			catch(e)
			{
				return;
			}
		}
	}

	/**
	 * Method: sweepOut
	 *
	 *	Gets the contents of the cell.
	 *
	 *	If the contents of the cell is simply HTML, it is returned, otherwise
	 *	in case the cell has children, their sweepOut() returned values are
	 *	grouped in a <thyCollection> and returned.
	 *
	 */
	p.sweepOut = function ()
	{
		if (!this.children.length) return this.getHTML();
		try
		{
			var i,contents = new thyCollection();
			for (i=0; i<this.children.length; i++)
			{
				contents.add(this.children[i].sweepOut(), this.children[i].name);
			}
			return contents;
		}
		catch(e)
		{
			return;
		}
	}

	p._thyPanelAddChild = p.addChild;
	/**
	 * Method: addChild
	 *
	 *	Adds a widget inside the cell
	 *
	 */
	p.addChild = function (child)
	{
		if (this.children.length == 0) this._thyPanelSetHTML('');
		this._thyPanelAddChild(child);
	}
	
	p._thyPanelSetHTML = p.setHTML;
	/**
	 * Method: setHTML
	 *
	 *	Overloaded method that sets HTML of Contents, never putting empty HTML
	 *	inside box (otherwise it would not be visible).
	 *
	 * Parameter:
	 *
	 *	html - The HTML code to be inserted
	 *
	 */
	p.setHTML = function (html)
	{
		var rSpace = new RegExp(' ', 'g');
		if (!html && typeof(html) != 'number') html = '&nbsp;';
		else if (typeof(html) == 'string') html = html.replace(rSpace, '&nbsp;');
		this._thyPanelSetHTML(html);
	}

	p._thyPanelGetHTML = p.getHTML;
	/**
	 * Method: getHTML
	 *
	 *	Overloaded method that returns the HTML of Contents and not the HTML
	 *	of cell. Besides that, it garantees that the return is not '&nbsp;', which
	 *	is set as HTML when setHTML receives a null parameter.
	 *
	 */
	p.getHTML = function ()
	{
		var rSpace = new RegExp('&nbsp[;]', 'g');
		var html = this._thyPanelGetHTML();
		
		if (html == '&nbsp' || html == '&nbsp;')
		{
			return '';
		}
		
		if (typeof(html) == 'string')
		{
			return html.replace(rSpace, ' ');
		}
		
		if (typeof(html) == 'object' && typeof(html.isFromClass) && html.isFromClass('thyPanel'))
		{
			return html.sweepOut();
		}
		
		return html;
	}
	
	/**
	 * Method: setMode
	 *
	 * Sets the onclick exchange mode of the grid. This can be:
	 * 
	 * - r (Read Only)
	 * - rws (Simple Read/Write)
	 * - rwf (Full Read/Write)
	 *
	 * Parameters:
	 *
	 *	mode - The mode
	 *
	 * 	- *Note* - The only supported mode now is Read Only
	 *
	 */
	p.setMode = function(mode)
	{
		if (this.cellType == 1) return;
		if (this.readOnly) return;

		switch (mode)
		{
			case 'r'  : mode = 0; break;
			case 'rws': mode = 1; break;
			case 'rwf': mode = 2; break;
			default: return;
		}

		this.cellMode = mode;
		this._processMode();
	}

	/**
	 * Method: setType
	 *
	 * Change this object type to one of:
	 *
	 * - Normal
	 * - Header
	 *
	 * Parameters:
	 *
	 *	type - The type. 'n' for Normal, 'h' for header
	 *
	 */
	p.setType = function(type)
	{
		switch (type)
		{
			case 'n': type = 0; break;
			case 'h': type = 1; break;
			default : return;
		}

		this.cellType = type;
		this._processType();
	}

	/**
	 * Method: setSelected
	 *
	 * Sets the selected state of this Cell
	 *
	 * Parameters:
	 *
	 *	state - true/false
	 *
	 */
	p.setSelected = function(state)
	{
		state = (state) ? true : false;
		if (this.cellSelected == state) return;
		this.cellSelected = state;
		this._processType();
	}

	/**
	 * Method: switchSelectedState
	 *
	 * Switches the actual selected state, turning the cell into selected if it is 
	 * not and vice-versa
	 *
	 */
	p.switchSelectedState = function()
	{
		this.setSelected(!this.cellSelected);
	}

	/**
	 * Method: isSelected
	 *
	 *	Returns the selected state of this Cell
	 *
	 * Returns:
	 *
	 *	Boolean
	 */
	p.isSelected = function()
	{
		return this.cellSelected;
	}

	/**
	 * Method: getOuterHTML
	 *
	 *	Overwritten method that draws TD element instead of DIV
	 *
	 */
	p.getOuterHTML = function ()
	{
		var html = [
			'<td '+this._cssClass+' id="'+this.id+'" '+(this.disabled ? 'disabled' : '')+' '+(this.readOnly?'readonly':'')+' style="',
			((this.x!=null)? ' left: '+((typeof(this.x)=='string')?this.x+';':this.x+'px;') : ''),
			((this.y!=null)? ' top: '+((typeof(this.y)=='string')?this.y+';':this.y+'px;') : ''),		
			((this.w)? ' width:'+((typeof(this.w)=='string')?this.w+';':(this.w+fixBw)+'px;'):''),
			((this.h)? ' height:'+((typeof(this.h)=='string')?this.h+';':(this.h+fixBh)+'px;'):''),
			((this.z)? ' z-index: '+this.z+';':''),
			((this._cursor!=null)? ' cursor:'+this._cursor+';':''),
			((this.visible==false)? ' visibility:hidden;': ''),
			((this.display) ? ' display: '+this.display+';' : ''),
			this._cssBorder,
			((this._overflow)? this._cssOverflow : ''),
			((this.x != null || this.y != null) ? this._cssPosition+';' : ''),
			(this._float ? this._cssFloat : ''),
			//(this.contentRelative ? '"><div class="'+this.name+'_relative'+'" style="position: relative; width: 100%; height: 100%">' : '">'),
			//(this.contentRelative ? '"><div class="'+this.name+'_relative'+'" style="">' : '">'),
			'">',
			this.getInnerHTML(),
			//(this.contentRelative ? '</div></div>' : '</div>')
			'</td>'
		].join('');
		//alert(html);
		return html;
	}

	/*************************************************************************\
	 *                          Private Methods                              *
	\*************************************************************************/
	
	p._create = function ()
	{
		if (this.parent && !this.elm)
		{
			var elm = this._createDOMElement();
			DynLayer._assignElement(this,elm);
			DynElement._flagCreate(this);
		}
	}

	p._createDOMElement = function ()
	{
		if (this.parent)
		{
			var elm, parentElm;

			parentElm = this.parent.elm;
			
			if (!this.elm)
			{
				elm = this.parent.doc.createElement('td');
				elm._className = this._cssClass;
				elm.id = this.id;
				var css = elm.style;
				if (this.w) css.width = this.w;
				if (this.h) css.height = this.h;
				if (this.bgColor != null) css.backgroundColor = this.bgColor;
				if (this.z) css.zIndex = this.z;
				if (this._cursor != null) css.cursor = this._cursor;
				if (this._overflow) css.overflow = this._overflow;
				css.visibility=(this.visible==false)? 'hidden':'';
				if(this.display) css.display = this.display;
				if(this.disabled) elm.disabled = true;

				elm.innerHTML = this.getInnerHTML();
			}
			else
			{
				elm = this.elm;
			}

			parentElm.appendChild(elm);
			return elm;
		}
	}
	
	/**
	 * Method: _processMode
	 *
	 * Process the modifications needed to achieve the selected mode.
	 *
	 */
	p._processMode = function()
	{
		switch (this.cellMode)
		{
			case 0:
				this.setHTML(this.cellEditBox.sweepOut());
				return;
				
			case 1:
				this.cellEditBox.populate(this.getHTML());
				this.cellEditBox.setDisplay(true);
				this.cellEditBox.focus();
		}
		
		return;
	}

	/**
	 * Method: _processType
	 *
	 * Process the modifications needed to achieve the type mode.
	 */
	p._processType = function()
	{
		switch (this.cellType)
		{
			case 0:
				this.setCSSClass(this.cellSelected ? 'selected' : 'default');
				return;

			case 1:
				this.setCSSClass(this.cellSelected ? 'header_selected' : 'header');
				return;
		}
	}

	/**
	 * Method: _selectEvent
	 *
	 *	Method that handles onclick events
	 *
	 */
	p._selectEvent = function()
	{
		this.setSelected(!this.cellSelected);
	}
