 /***************************************************************************\
 * ThyAPI - Thyamad Javascript API - ListBox Element                         *
 * http://www.thyamad.com                                                    *
 *                                                                           *
 * Copyright (C) 2005 - Raphael Derosso Pereira                              *
 * Based on DynAPI v3.0b1                                                    *
 * ------------------------------------------------------------------------- *
 *  This library is free software; you can redistribute it and/or modify it  *
 *  under the terms of the GNU Lesser General Public License as published    *
 *  by the Free Software Foundation; either version 2.1 of the License,      *
 *  or any later version.                                                    *
 *  This library is distributed in the hope that it will be useful, but      *
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *  See the GNU Lesser General Public License for more details.              *
 \***************************************************************************/


	/**
	 * Class: thyListBox
	 *
	 *	This is the ListBox Element, derivate from thyGridPanel.
	 *
	 *	The *thyListBox* permits the user to do all things a common ListBox
	 *	allows, but differently from other ListBoxes, thyListBox has 3 columns:
	 *	
	 *	- The first shows the index;
	 *	- The second is for value/id;
	 *	- The third is for contents.
	 *
	 *	If you don't want to show indexes, set .thyListBox_col-1 CSS class width
	 *	property to 0px. The same applies to col0, which is id/value. If you do 
	 *	that you'd have a common listbox.
	 *
	 * CSS Classes:
	 *
	 *	- .thyListBox_row
	 *	- .thyListBox_selected_row
	 *	- .thyListBox_header_row
	 *	- .thyListBox_header_selected_row
	 *
	 *	- .<object name>_row
	 *	- .<object name>_header_row
	 *	- .<object name>_selected_row
	 *	- .<object name>_selected_header_row
	 *
	 * See Also:
	 *
	 *	- <thyGridCell>
	 *	- <thyCollection>
	 *	- <thyPanel>
	 *
	 */
	function thyListBox(name, label)
	{
		this.thyGrid = thyGrid;
		this.thyGrid(name,label);

		// Atributes
		this._thyGridAppendColumn('&nbsp;');
		this._thyGridAppendColumn('&nbsp;');
		
		// Initialization
		this.addCSSClass('thyListBox');
		this.setSelectCellSelectRow();
	}

	p = dynapi.setPrototype('thyListBox', 'thyGrid');

	/**
	 * Method: cleanUp
	 *
	 *	Removes all rows from list
	 *
	 */
	p.cleanUp = function ()
	{
		this.removeAllRows();
	}
	
	/**
	 * Method: reset
	 *
	 *	Unselect all rows
	 *
	 */
	p.reset = function ()
	{
		var rows = this.getSelectedRowsIndexes();
		var i, nRows = rows.length();

		for (i=0; i<nRows; i++)
		{
			this.getRowElement(rows.i(i)).unselect();
		}
	}
	
	/**
	 * Method: insertColumn
	 *
	 *	Overloaded method that prevents the insertion of columns in a listbox
	 *
	 */
	p.insertColumn = function ()
	{
		throw({result: 'Adding Column in ListBox', location: 'thyListBox.insertColumn'});
	}

	p._thyGridAppendColumn = p.appendColumn;
	/**
	 * Method: appendColumn
	 *
	 *	Overloaded method that prevents the insertion of columns in a listbox
	 *
	 */
	p.appendColumn = function ()
	{
		throw({result: 'Adding Column in ListBox', location: 'thyListBox.appendColumn'});
	}

	/**
	 * Method: appendRow
	 *
	 *	Overloaded method that garantees if the user sends just one field
	 *	for each roll, they will be assumed as contents and their values
	 *	will be set to null
	 *
	 */
	p._thyGridAppendRow = p.appendRow;
	p.appendRow = function (row)
	{
		if (!row || (row._className != 'thyCollection' && !row.thyCollection)) throw({result: 'Invalid Collection', location: 'thyGrid.appendRow'});

		var nCells = row.getElementsCount();

		if (nCells < 2)
		{
			row.add('',null,0);
		}

		return this._thyGridAppendRow(row);
	}

	/**
	 * Method: getSelectedValues
	 *
	 *	Returns an array with the values of the selected rows
	 *
	 */
	p.getSelectedValues = function ()
	{
		var selRows = this.getSelectedRows();
		var i,nRows = selRows.length();
		var selectedValues = [];

		for (i=0; i<nRows; i++)
		{
			selectedValues[i] = selRows.i(i).i(0);
		}

		return selectedValues;
	}

	/**
	 * Method: getSelectedIndexes
	 *
	 *	Returns an array with the indexes of the selected rows
	 *
	 */
	p.getSelectedIndexes = function ()
	{
		return this.getSelectedRowsIndexes().getIndexedElements();
	}

	/**
	 * Method: getSelected
	 *
	 *	Returns all selected informations
	 *
	 *	See <thyGrid::getSelectedRows> for more information
	 *
	 */
	p.getSelected = p.getSelectedRows;

	/**
	 * Method: removeSelected
	 *
	 *	Removes all selected rows
	 *
	 *	See <thyGrid::removeSelectedRows> for more information
	 *
	 */
	p.removeSelected = p.removeSelectedRows;

	/**
	 * Method: getContentByValue
	 *
	 *	Returns the content associated with the given value
	 *
	 * Parameter:
	 *
	 *	value - The value that is associated with a content
	 *
	 */
	p.getContentByValue = function (value)
	{
		var nRows = this.getRowsCount();

		for (i=0; i<nRows; i++)
		{
			if (this.getRowElement(i).getCellElement(0).sweepOut() == value)
				return this.getRowElement(i).getCellElement(1).sweepOut();
		}
	}

	/**
	 * Method: search
	 *
	 *	Searches for a specified string inside values column
	 *
	 * Parameter:
	 *
	 *	str - The string to be searched for
	 *
	 */
	p.search = function (str)
	{
		return this.searchColumn(str,1);
	}

	/*************************************************************************\
	 *                          Private Methods                              *
	\*************************************************************************/

	p._populateContents = function (data)
	{
		var i, j, row, cell;

		if (!data.selectedValues || data.rows)
		{
			var rows;
			if (data.rows) rows = data.rows;
			else rows = data;

			this.removeAllRows();
			
			for (i = 0; i<rows.length; i++)
			{
				if (typeof(rows[i]) != 'object') rows[i] = [rows[i]];
				row = new thyCollection('Row'+i);

				for (j=0; j<rows[i].length; j++)
				{
					row.add(rows[i][j],'Cell'+i+j);
				}
				this.appendRow(row);
			}
		}

		if (data.selectedValues)
		{
			var values = data.selectedValues;
			var nValues = data.selectedValues.length;
			var nRows = this.getRowsCount();

			for (i=0; i<nValues; i++)
			{
				for (j=0; j<nRows; j++)
				{
					if (this.getRowElement(j).getCellElement(0).sweepOut() == values[i])
						this.getRowElement(j).select();
				}
			}
		}
	}

	/**
	 * Method: _sweepOutContents
	 *
	 *	Return selected values
	 *
	 */
	p._sweepOutContents = function ()
	{
		var selRows = this.getSelectedRows();
		var i,nRows = selRows.length();
		var data = {
			selectedValues: this.getSelectedValues(), 
			selectedIndexes: this.getSelectedIndexes()
		};

		return data;
	}
	

