 /***************************************************************************\
 * ThyAPI - Thyamad Javascript API - Calendar Element                        *
 * http://www.thyamad.com                                                    *
 *                                                                           *
 * Copyright (C) 2005 - Raphael Derosso Pereira                              *
 * Based on DynAPI v3.0b1                                                    *
 * Based on JSCalendar by Mihai Bazon, <mihai_bazon@yahoo.com>               *
 * ------------------------------------------------------------------------- *
 *  This library is free software; you can redistribute it and/or modify it  *
 *  under the terms of the GNU Lesser General Public License as published    *
 *  by the Free Software Foundation; either version 2.1 of the License,      *
 *  or any later version.                                                    *
 *  This library is distributed in the hope that it will be useful, but      *
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *  See the GNU Lesser General Public License for more details.              *
 \***************************************************************************/


	if (!window.Calendar) dynapi.library.include('JSCalendarSetup');

	/**
	 * Class: thyPopupCalendar
	 *
	 *	This is the PopupCalendar class. It is wrapper to use JS Calendar.
	 *	
	 *	Thanks for Mihai Bazon <mihai_bazon@yahoo.com> for the great work!
	 *	For original JS Calendar, access <http://dynarch.com/mishoo/>
	 *
	 *	This widget is actually just a wrapper that puts JS Calendar into a
	 *	thyPanel and makes it compatible with thyAPI.
	 *
	 * CSS Classes:
	 *
	 *	- .thyPopupCalendar
	 *
	 *	- .<object name>
	 *
	 * Events:
	 *
	 *	onchange - invoked when the date is changed
	 *
	 * See Also:
	 *
	 *	- <thyEditBox>
	 *
	 */
	function thyPopupCalendar(name)
	{
		this.thyEditBox = thyEditBox;
		this.thyEditBox(name);

		//** Attributes **\\
		this.timestamp = null;
		this.format = '%Y/%m/%d';
		this.calBtn = new thyButton(this.name+'_button');

		this.calBtn.addCSSClass('thyPopupCalendar_button');
		
		this.addChild(this.calBtn);

		this.readOnly = true;
		
		this.onCreate(this.initThyPopupCalendar);
	}

	p = dynapi.setPrototype('thyPopupCalendar','thyEditBox');

	/**
	 * Method: initThyPopupCalendar
	 *
	 *	Initialization Method
	 *
	 */
	p.initThyPopupCalendar = function ()
	{
		Calendar.setup({
			inputField: this.name+'_input',
			ifFormat: this.format,
			button: this.calBtn,
			onSelect: this._calendarOnSelect(),
			thyAPIObj: this
		});
		
		var self = this;
		this.calBtn.addEventListener({onclick: function()
		{
			var cal = window.calendar;
			if (cal)
			{
				cal.element.style.zIndex = 10000000;
			}
		}});
	}

	/**
	 * Method: setFormat
	 *
	 *	Sets the default format for displaying
	 *
	 * Parameters:
	 *
	 *	format - The format
	 *
	 */
	p.setFormat = function (format)
	{
		if (this.timestamp != null) this.populate(this.getTimestamp());

		if (this._created)
		{
			window.calendar.setDateFormat(format);
			return;
		}

		this.format = format;
	}

	/**
	 * Method: getFormatted
	 *
	 *	Returns the actual timestamp formatted with the actual
	 *	format string
	 *
	 */
	p.getFormatted = function ()
	{
		var date = new Date();
		date.setTime(this.timestamp);
		return date.print(this.format);
	}

	/**
	 * Method: getTimestamp
	 *
	 *	Return the timestamp of this field
	 *
	 * Parameter:
	 *
	 *	miliseconds - Returns miliseconds timestamp instead of seconds
	 *
	 */
	p.getTimestamp = function (miliseconds)
	{
		if (miliseconds) return this.timestamp;
		else return Math.ceil(this.timestamp/1000);
	}

	/*************************************************************************\
	 *                        Group: Private Methods                         *
	\*************************************************************************/

	p._thyEditBox_setReadOnly = p._setReadOnly;
	/**
	 * Method: _setReadOnly
	 *
	 *	Turns the EditBox into ReadOnly mode
	 *
	 */
	p._setReadOnly = function ()
	{
		// Do Nothing, as this object's editbox IS read-only and can be set only
		// using the popup calendar
	}
	
	p._thyEditBox_populateContents = p._populateContents;
	/**
	 * Method: _populateContents
	 *
	 *	Overloaded method that sets the field using timestamp and
	 *	format.
	 *
	 */
	p._populateContents = function (data)
	{
		if (!data) return;
		
		this.timestamp = data*1000;
		this._thyEditBox_populateContents(this.getFormatted());
	}

	/**
	 * Method: _sweepOutContents
	 *
	 *	Overloaded method that returns the timestamp
	 *
	 */
	p._sweepOutContents = function ()
	{
		return this.getTimestamp();
	}
	
	/**
	 * Method: _calendarOnSelect
	 *
	 *	Event that handles clicks on dates. This is the original
	 *	onSelect event distributed with JSCalendar, but adapted to
	 *	save timestamp information to this class.
	 *
	 */
	p._calendarOnSelect = function ()
	{
		var self = this;

		return function (cal)
		{
			var p = cal.params;
			var update = (cal.dateClicked || p.electric);

			if (update && p.inputField) 
			{
				p.inputField.value = cal.date.print(p.ifFormat);

				// thyAPI functions
				self.timestamp = cal.date.getTime();
				self.invokeEvent('change',new DynEvent('change',self));
			}

			if (update && p.singleClick && cal.dateClicked)
				cal.callCloseHandler();
		};
	}

