 /***************************************************************************\
 * ThyAPI - Thyamad Javascript API - Rich Text Editor Element                *
 * http://www.thyamad.com                                                    *
 *                                                                           *
 * Copyright (C) 2005 - Raphael Derosso Pereira                              *
 * Based on DynAPI v3.0b1                                                    *
 * ------------------------------------------------------------------------- *
 *  This library is free software; you can redistribute it and/or modify it  *
 *  under the terms of the GNU Lesser General Public License as published    *
 *  by the Free Software Foundation; either version 2.1 of the License,      *
 *  or any later version.                                                    *
 *  This library is distributed in the hope that it will be useful, but      *
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *  See the GNU Lesser General Public License for more details.              *
 \***************************************************************************/


	/**
	 * Class: thyRichTextEdit
	 *
	 *  This class is a wrapper to permit ThyAPI applications/widgets to use
	 *  the power of an existing Rich Text Editor,
	 *  FCKeditor <http://www.fckeditor.net/> (Thanks *Frederico* for the great
	 *  work!!!).
	 *
	 *  It encapsulates all FCKEditor functions/methods in a ThyAPI like
	 *  structure. This garantees compatibility and makes it easier to manage
	 *  releases and upgrades, and also incorporate ThyAPI design concept, that
	 *  is to keep all visual elements on CSS.
	 *
	 *  Unfortunatly this version of thyRichTextEdit doesn't permit the designer
	 *  to edit FCKEditor style within its CSS classes... maybe in future
	 *  versions
	 *
	 */
	function thyRichTextEdit(name, html)
	{
		this.thyLabelPanel = thyLabelPanel;
		this.thyLabelPanel(name);

		//** Attributes **\\
		this.contentsHTML = html;
		this.addCSSClass('thyRichTextEdit');

		//** Initialization **\\
		//if (!this.thyRichTextEdit) this.onPreCreate(this.initThyRichTextEdit);
		this.onPreCreate(this.initThyRichTextEdit);
	}

	p = dynapi.setPrototype('thyRichTextEdit', 'thyLabelPanel');

	/**
	 * Method: initThyRichTextEdit
	 *
	 *	Initialization Method
	 *
	 */
	p.initThyRichTextEdit = function ()
	{
		this.FCKeditor = new FCKeditor(this.name+'_editor', '100%', '100%');
		
		var self = this;
		this.disabledPanel = new thyPanel();

		// Initialization
		this.disabledPanel.addCSSClass('thyLabelPanel_disabled');
		this.setRelativeness(false);
		this.FCKeditor.BasePath = dynapi.library.path+'/thywidgets/external/fckeditor/';
		this.FCKeditor.ToolbarSet = 'Basic';
		this.FCKeditor.Value = this.contentsHTML || '';
		this.fckElm;

		this.contents.setHTML(this.getContentsInnerHTML());
		
		// Initialization
		this.onCreate(this._eventOnCreate());
		
		this.addEventListener({
			onmouseover: function(e)
			{
				if (self.fckElm.MakeEditable && !self.disabled) self.fckElm.MakeEditable();
			}
		});

		this.contents.invokeEvent = function()
		{
			return;
		}
	}

	/**
	 * Method: sweepOut
	 *
	 *	Returns the contents of the Rich Text Editor
	 *
	 */
	p.sweepOut = function ()
	{
		return (this._created && this.fckElm) ? this.fckElm.GetHTML() : this.FCKeditor.Value;
	}

	p._thyLabelPanelAddChild = p.addChild;
	/**
	 * Method: addChild
	 *
	 *	Overwritten method that impossibilitates the developer to add childs to
	 *	this widget, as this have no meaning.
	 *
	 */
	p.addChild = function ()
	{
		throw({ result: 'No child to be added', location: 'thyRichTextEditor.addChild' });
	}

	/**
	 * Method: removeChild
	 *
	 *	Overwritten method that impossibilitates the developer to remove childs from
	 *	this widget, as this have no meaning
	 *
	 */
	p.removeChild = function ()
	{
		throw({ result: 'No child to be removed', location: 'thyRichTextEditor.removeChild' });
	}
	
	/**
	 * Method: setHTML
	 *
	 *	Overwritten method that makes setHTML to have the same functionality as
	 *	populate
	 *
	 */
	p.setHTML = function (html)
	{
		this.populate(html);
	}

	p._uberSetDisplay = p.setDisplay;
	/**
	 * Method: setDisplay
	 *
	 *	FCK Editor doesn't like 'display: none' property.
	 *
	 *	This overloaded method garantees that on comming back from 'display: none'
	 *	the Editor is editable
	 *
	 */
	p.setDisplay = function (b)
	{
		this._uberSetDisplay(b);
		
		if (b && this.fckElm && this.fckElm.MakeEditable) 
			this.fckElm.MakeEditable();
	}

	/*************************************************************************\
	 *                       Group: Private Methods                          *
	\*************************************************************************/

	/**
	 * Method: _cleanUp
	 *
	 *	Removes all data from rich text
	 *
	 */
	p._cleanUp = function ()
	{
		this.populate('');
	}

	/**
	 * Method: _reset
	 *
	 *	Removes all data from rich text
	 *
	 */
	p._reset = p._cleanUp;
	
	/**
	 * Method: _populateContents
	 *
	 *	Sets the contents of the Rich Text Editor
	 *
	 */
	p._populateContents = function (data)
	{
		if (this._created && this.fckElm)
		{
			this.fckElm.SetHTML(data);
		}
		else
		{
			this.FCKeditor.Value = data;
		}
	}
	
	/**
	 * Metod: _setDisabled
	 *
	 *	Disable/Enable Editor
	 *
	 */
	p._setDisabled = function ()
	{
		if (!this._created || !this.fckElm) return;
		
		if (this.disabled)
		{
			//this.disabledPanel.setX(this.contents.getAbsoluteX());
			//this.disabledPanel.setY(this.contents.getAbsoluteY());
			this.disabledPanel.setX(0);
			this.disabledPanel.setY(0);
			this.disabledPanel.setSize(this.contents.getWidth(), this.contents.getHeight());
			this.disabledPanel.setZIndex({above: this.contents});
			this.contents.addChild(this.disabledPanel);
			//this.fckElm.EditorDocument.designMode = 'off';
			//this.fckElm.EditorWindow.blur();
		}

		if (!this.disabled)
		{
			this.contents.removeChild(this.disabledPanel);
			if (this.fckElm.MakeEditable) this.fckElm.MakeEditable();
		}
	}

	/**
	 * Method: getInnerHTML
	 *
	 *	Returns the HTML needed to insert FCK editor inside the thyAPI panel
	 *	this class holds
	 *
	 */
	p.getContentsInnerHTML = function ()
	{
		var html = '';
		if ( !this.FCKeditor.CheckBrowser || this.FCKeditor._IsCompatibleBrowser() )
		{
			html += '<input type="hidden" id="' + this.FCKeditor.InstanceName + '" name="' + this.FCKeditor.InstanceName + '" value="' + this.FCKeditor._HTMLEncode( this.FCKeditor.Value ) + '" />';
			html += this.FCKeditor._GetConfigHtml();
			html += this.FCKeditor._GetIFrameHtml();
		}
		else
		{
			var sWidth  = this.FCKeditor.Width.toString().indexOf('%')  > 0 ? this.FCKeditor.Width  : this.FCKeditor.Width  + 'px' ;
			var sHeight = this.FCKeditor.Height.toString().indexOf('%') > 0 ? this.FCKeditor.Height : this.FCKeditor.Height + 'px' ;
			html += '<textarea name="' + this.FCKeditor.InstanceName + '" rows="4" cols="40" style="WIDTH: ' + sWidth + '; HEIGHT: ' + sHeight + '" wrap="virtual">' + this.FCKeditor._HTMLEncode( this.FCKeditor.Value ) + '<\/textarea>';
		}

		return html;
	}

	/*************************************************************************\
	 *                            Group: Events                              *
	\*************************************************************************/

	/**
	 * Method: _eventOnCreate
	 *
	 *	Get the FCK element and perform some other operations upon creation
	 *
	 */
	p._eventOnCreate = function ()
	{
		var self = this;
		return function ()
		{
			var time = 100;
			var grabbed = false;
			var grabFCK = function()
			{
				if (!grabbed)
				{
					if (!window.FCKeditorAPI || !window.FCKeditorAPI.GetInstance(self.name+'_editor'))
					{
						setTimeout(grabFCK, time);
						return;
					}

					self.fckElm = window.FCKeditorAPI.GetInstance(self.name+'_editor');
				}
				
				if (!self.fckElm.EditorDocument)
				{
					grabbed = true;
					setTimeout(grabFCK, time);
					return;
				}

				self._setDisabled();
			}

			grabFCK();
		}
	}
