 /***************************************************************************\
 * ThyAPI - Thyamad Javascript API - Tabs Manager Element                    *
 * http://www.thyamad.com                                                    *
 *                                                                           *
 * Copyright (C) 2005 - Raphael Derosso Pereira                              *
 * Based on DynAPI v3.0b1                                                    *
 * ------------------------------------------------------------------------- *
 *  This library is free software; you can redistribute it and/or modify it  *
 *  under the terms of the GNU Lesser General Public License as published    *
 *  by the Free Software Foundation; either version 2.1 of the License,      *
 *  or any later version.                                                    *
 *  This library is distributed in the hope that it will be useful, but      *
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *  See the GNU Lesser General Public License for more details.              *
 \***************************************************************************/

	/**
	 * Class: thyTabsManager
	 *
	 *	Tabs Manager Class.
	 *
	 *	This class implements the TabsManager Element. This element handles 
	 *	tabs contents.
	 *
	 * CSS Classes:
	 *
	 *	- .thyTabsManager_contents
	 *	- .thyTabsManager_tabsPanel
	 *	- .thyTabsManager_control - All tab controls
	 *	- .thyTabsManager_control_active
	 *	- .thyTabsManager_control_inactive
	 *
	 *	- .<object name>_contents
	 *	- .<object name>_tabsPanel
	 *	- .<object name>_control
	 *	- .<object name>_control_active
	 *	- .<object name>_control_inactive
	 *
	 *
	 * Events:
	 *
	 *	onchangetab - Invoked when Active tab is changed
	 *
	 * Author:
	 *
	 *	Raphael Derosso Pereira <raphaelpereira@users.sourceforge.net>
	 *
	 */
	function thyTabsManager(name)
	{
		this.thyPanel = thyPanel;
		this.thyPanel(name);

		// Attributes
		this.tabs;
		this.tabsCount = 0;
		this.tabsPosition;
		this.activeTab;
		this.tabsContents = new thyPanel(this.name+'_contents');
		this.tabsPanel  = new thyPanel(this.name+'_tabsPanel');

		this.addCSSClass('thyTabsManager');
		
		this._thyPanelAddChild(this.tabsPanel);
		this._thyPanelAddChild(this.tabsContents);
		
		this.onPreCreate(this.initThyTabsManager);
	}

	p = dynapi.setPrototype('thyTabsManager', 'thyPanel');

	/**
	 * Method: initThyTabsManager
	 *
	 *	Initialization Method
	 *
	 */
	p.initThyTabsManager = function ()
	{
		// Content Holder Initialization
		this.tabsContents.setX(0);
		this.tabsContents.setY(0);
		this.tabsContents.setPosition('relative');
		this.tabsContents.setOverflow('hidden');

		// Tabs Panel Initialization
		this.tabsPanel.setOverflow('auto');
		this.tabsPanel.setZIndex({above: this.tabsContents});
	}
	
	/**
	 * Method: populate
	 *
	 *	Used to set Caption
	 *
	 */
	p.populate = function (data)
	{
		var i, info;

		for (i in data)
		{
			if (data[i]['caption'])
			{
				this.setCaption(i,data[i]['caption']);
				if (!data[i]['contents']) continue;
				info = data[i]['contents'];
			}
			else if (data[i]['contents'] && this.tabs[i])
			{
				info = data[i]['contents'];
			}
			else info = data[i];
			
			this.tabs[i].content.populate(info);
		}
	}

	/**
	 * Method: sweepOut
	 *
	 *	Returns the contents of each tab
	 *
	 */
	p.sweepOut = function ()
	{
		var i, result = {};
		
		for (i in this.tabs)
		{
			result[i] = this.tabs[i].content.sweepOut();
		}

		return result;
	}

	/**
	 * Method: setDisabled
	 *
	 *	Overloaded method that calls just the tabs contents setDisabled method
	 *
	 */
	p.setDisabled = function (bool)
	{
		var i,nc = this.tabsContents.children.length;
		
		for (i=0; i<nc; i++)
		{
			if (this.tabsContents.children[i].setDisabled) this.tabsContents.children[i].setDisabled(bool);
		}
	}

	/**
	 * Method: reset
	 *
	 *	Overloaded method that resets just TabsContents
	 *
	 */
	p.reset = function ()
	{
		this.tabsContents.reset();
		if (this._reset) this._reset();
	}
	
	p._thyPanelSetName = p.setName;
	/**
	 * Method: setName
	 *
	 *	Overwritten method that sets the name of main, contents and tabs panel
	 *	and refreshes CSS Classes
	 *
	 * Parameter:
	 *
	 *	name - The new widget name
	 *
	 */
	p.setName = function (name)
	{
		if (name == null || this.name == name) return;
		
		this._thyPanelSetName(name);
		if (this.tabsContents) this.tabsContents.setName(name+'_contents');
		if (this.tabsPanel) this.tabsPanel.setName(name+'_tabsPanel');
	}
	 
	p._thyPanelSetCSSClass = p.setCSSClass;
	/**
	 * Method: setCSSClass
	 *
	 *	Overwritten <thyPanel::setCSSClass> method that sets the class of
	 *	main, tabs panel and contents
	 *
	 * Parameter:
	 *
	 *	type - The class type
	 *
	 */
	p.setCSSClass = function (type)
	{
		this._thyPanelSetCSSClass(type);
		if (this.tabsContents) this.tabsContents.setCSSClass(type);
		if (this.tabsPanel) this.tabsPanel.setCSSClass(type);
	}
	
	/**
	 * Method: setCaption
	 *
	 *	Sets the caption of a tab.
	 *
	 * Parameters:
	 *
	 *	tab - The tab name
	 *	caption - The caption to be set
	 *
	 */
	p.setCaption = function (tab, caption)
	{
		if (typeof(tab) == 'string')
		{
			if (!(tab = this.tabs[tab])) return;
		}

		tab.control.setHTML(caption);
	}
	 
	/**
	 * Method: addTab
	 *
	 *	Adds a Panel as a tab contents.
	 *
	 * Parameters:
	 *
	 *	caption - The text that will be shown on the tab switch
	 *	tab - The tab itself. This must be a thyPanel
	 *
	 */
	p.addTab = function (tab, caption)
	{
		if (!tab.DynLayer) return;
		
		var activate = false;
		if (!this.tabsCount)
		{
			this.tabs = {};
			activate = true;
		}
		
		var name = tab.name;
		var self = this;
		var tabControl = new thyPanel(name, caption);
		tabControl.addCSSClass('thyTabsManager');
		tabControl.cssClasses = {
			inactive : ['control','control_inactive'],
			active   : ['control','control_active']
		};
		tabControl.setRelativeness(false);
		tabControl.setTextSelectable(false);
		tabControl.setCSSClass('inactive');
		tabControl.setCursor('hand');
		tabControl.setZIndex({above: this.tabsContents});
		tabControl.addEventListener({onclick: function(e){
			self.setActive(name);
		}});
		
		tab.setX(0);
		tab.setY(0);
		tab.setSize('100%', '100%');
		//tab.setOverflow('auto');
		tab.setPosition('relative');
		tab.setDisplay(false);
		tab.setZIndex({below: tabControl});
		
		this.tabsContents.addChild(tab);
		this.tabsPanel.addChild(tabControl);
		this.tabs[name] = { control: tabControl, content: tab};
		
		if (activate) this.setActive(tab);
		this.tabsCount++;
	}

	/**
	 * Method: removeTab
	 *
	 *	Removes the specified tab from Tabs Manager
	 *
	 * Paramenter:
	 *
	 *	tab - The thyPanel that was previously inserted as a tab
	 *
	 */
	p.removeTab = function(tab)
	{
		var i,name = tab.name;
		if (!this.tabs || !this.tabs[name]) return;
		
		tab = this.tabs[name];
		if (this.activeTab == tab)
			for (i in this.tabs) 
			{
				if (this.tabs[i] == tab) continue;

				this.setActive(name);
				break;
			}
		
		this.tabsContents.removeChild(this.tabs[name].content);
		this.tabsPanel.removeChild(this.tabs[name].control);
		this.tabs[name] = null;
		this.tabsCount--;
	}

	/**
	 * Method: setActive
	 *
	 *	Changes the specified Tab to active state
	 *
	 * Parameter:
	 *
	 *	tab - Can be the <thyPanel> or just the name of the tab that should 
	 *	      be activated
	 *
	 */
	p.setActive = function (tab)
	{
		if (typeof(tab) == 'string')
		{
			if (!(tab = this.tabs[tab])) return;
			tab = tab.content;
		}
		else if (typeof(tab) != 'object' || !tab.name || !this.tabs[tab.name])
		{
			return;
		}

		if (this.activeTab)
		{
			if (tab == this.activeTab.content) return;
		
			this.activeTab.control.setCSSClass('inactive');
			this.activeTab.content.setDisplay(false);
		}

		this.activeTab = this.tabs[tab.name];
		this.activeTab.control.setCSSClass('active');
		this.activeTab.content.setDisplay(true);

		this.invokeEvent('changetab', new DynEvent('tabchange', this));
	}

	p._thyPanelAddChild = p.addChild;
	/**
	 * Method: addChild
	 *
	 *	Overwritten method that turns addChild into an alias of addTab
	 *
	 * Parameters:
	 *
	 *	caption - The text that will be shown on the tab switch
	 *	tab - The tab itself. This must be a thyPanel
	 *
	 */
	p.addChild = function (caption, tab)
	{
		this.addTab(caption, tab);
	}

	p._thyPanelRemoveChild = p.removeChild;
	/**
	 * Method: removeChild
	 *
	 *	Overwritten method that turns removeChild into an alias of removeTab
	 *
	 * Paramenter:
	 *
	 *	tab - The thyPanel that was previously inserted as a tab
	 *
	 */
	p.removeChild = function(tab)
	{
		this.removeTab(tab);
	}
	
	/*************************************************************************\
	 *                       Group: Private Methods                          *
	\*************************************************************************/

	p._thyPanel_addCSSClass = p._addCSSClass;
	/**
	 * Method: _addCSSClass
	 *
	 *	Overwritten method that adds the class to main, contents and tabs panel
	 *
	 * Parameter:
	 *
	 *	cssClass - The class to be appended
	 *
	 */
	p._addCSSClass = function (cssClass)
	{
		this._thyPanel_addCSSClass(cssClass);
		if (this.tabsContents) this.tabsContents._addCSSClass(cssClass+'_contents');
		if (this.tabsPanel) this.tabsPanel._addCSSClass(cssClass+'_tabsPanel');
	}
	
	p._thyPanel_removeCSSClass = p.r_emoveCSSClass;
	/**
	 * Method: _removeCSSClass
	 *
	 *	Overwritten Method that removes the specified CSS Class from main, contents
	 *	and tabs panel
	 *
	 * Parameter:
	 *
	 *	cssClass - The CSS Class to be removed
	 *
	 */
	p._removeCSSClass = function (cssClass)
	{
		this._thyPanel_removeCSSClass(cssClass);
		if (this.tabsContents) this.tabsContents._removeCSSClass(cssClass+'_contents');
		if (this.tabsPanel) this.tabsPanel._removeCSSClass(cssClass+'_tabsPanel');
	}


