 /***************************************************************************\
 * ThyAPI - Thyamad Javascript API - Window Element                          *
 * http://www.thyamad.com                                                    *
 *                                                                           *
 * Copyright (C) 2005 - Raphael Derosso Pereira                              *
 * Based on DynAPI v3.0b1                                                    *
 * ------------------------------------------------------------------------- *
 *  This library is free software; you can redistribute it and/or modify it  *
 *  under the terms of the GNU Lesser General Public License as published    *
 *  by the Free Software Foundation; either version 2.1 of the License,      *
 *  or any later version.                                                    *
 *  This library is distributed in the hope that it will be useful, but      *
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *  See the GNU Lesser General Public License for more details.              *
 \***************************************************************************/

	/**
	 * Class: thyWindow
	 *
	 *	This class implements the infamous Window widget. As it is derivate 
	 *	from thyBorderPanel, it has borders and all the CSS classes that widget
	 *	has.
	 *
	 * CSS classes:
	 *
	 *	- .thyWindow_title
	 *	- .thyWindow_contents
	 *	- .thyWindow_btnClose
	 *	- .thyWindow_btnStick
	 *	- .thyWindow_ondrag
	 *
	 *	- .<object name>_title
	 *	- .<object name>_contents
	 *	- .<object name>_btnClose
	 *	- .<object name>_btnStick
	 *	- .<object name>_ondrag
	 *
	 * Author:
	 *
	 *	Raphael Derosso Pereira <raphaelpereira@users.sourceforge.net>
	 *
	 */
	function thyWindow(name,title,html)
	{
		this.thyBorderPanel = thyBorderPanel;
		this.thyBorderPanel(name);

		// Attributes
		this.openned = false;
		this.sticked = false;
		this.title = title || this.name;
		this.html = html;
		this.content    = new thyPanel(this.name+'_contents',this.html);
		this.titleLayer = new thyPanel(this.name+'_title', this.title);
		this.btnClose   = new thyPanel(this.name+'_btnClose');
		this.btnStick   = new thyPanel(this.name+'_btnStick');
		this.winDragBorder = new thyPanel(this.name+'_ondrag',this.title);

		this.cssClasses['window'] = ['window'];
		this.addCSSClass('thyWindow');
		this.content.cssExtraClasses   = this.cssExtraClasses;
		this.titleLayer.cssExtraClasses = this.cssExtraClasses;
		
		// Turn Window into Draggable element
		this.winDragBorder.setAnchor({
			topA: this.id, 
			leftA: this.id
		});
		this.winDragBorder.setRelativeness(false);
		this.winDragBorder.setDisplay(false);

		// Initialize Title Bar
		var self = this;
		this.titleLayer.setRelativeness(false);
		//this.titleLayer.setTextSelectable(false);

		// Initialize Close Button
		this.btnClose.setRelativeness(false);
		this.btnClose.addEventListener({
			onclick: function(e){self.close()}
		});

		// Initialize Stick Button
		this.btnStick.setRelativeness(false);
		this.btnStick.addEventListener({
			onclick: function(e){self.switchStick()}
		});

		// Initialize Content
		this.content.invokeEvent = function (type,e,args){dynapi.document.invokeEvent(type,e,args);};

		this._thyBorderPanelAddChild(this.titleLayer);
		this._thyBorderPanelAddChild(this.btnStick);
		this._thyBorderPanelAddChild(this.btnClose);
		this._thyBorderPanelAddChild(this.content);
		//this.setVisible(false);
		this.setDisplay(false);
		this.setSize(0,0);

		//if (!this.thyWindow) this.onPreCreate(this.initThyWindow);
		this.onPreCreate(this.initThyWindow);
		this.onCreate(function()
		{
		});
	}

	p = dynapi.setPrototype('thyWindow','thyBorderPanel');

	/**
	 * Method: initThyWindow
	 *
	 *	Initialization method
	 *
	 */
	p.initThyWindow = function ()
	{
		this.setDragEnabled(true,null,true);
		this.setDragIcon(this.winDragBorder);
		//dynapi.document.addChild(this.winDragBorder);
	}

	/**
	 * Method: populate
	 *
	 *	Sets the title of the widow
	 *
	 */
	p.populate = function (data)
	{
		if (data['title']) this.setTitle(data['title']);
	}
	
	/**
	 * Method: setTitle
	 *
	 *	Sets the Title of the window
	 *
	 */
	p.setTitle = function(title)
	{
		this.titleLayer.setHTML('');
		
		this.setSize('','');
		this.setDisplay(true);
		
		var w = this.titleLayer.getWidth();
		
		this.titleLayer.setHTML(title);

		var length = title.length;
		
		if (w >= this.titleLayer.getWidth())
		{
			this.winDragBorder.setHTML(title);
		}
		else
		{
			while (w < this.titleLayer.getWidth() && length > 0)
			{
				length--;
				this.titleLayer.setHTML(dynapi.functions.limitString(title,length));
			}
			
			this.titleLayer.setHTML(dynapi.functions.limitString(title,length-3));
		}
		
		if (!this.openned)
		{
			this.setDisplay(false);
			this.setSize(0,0);
		}
	}

	/**
	 * Method: getTitle
	 *
	 *	Returns the actual title set
	 *
	 */
	p.getTitle = function ()
	{
		return this.titleLayer.getHTML();
	}

	/**
	 * Method: open
	 *
	 *	UnHide the window
	 *
	 */
	p.open = function()
	{
		var self = this;

		this.setSize('','');
		this.setDisplay(true);

		var position = function()
		{
			var pos = thyWindow._getWindowPosition(self);
			self.setX(pos.x);
			self.setY(pos.y);
		};

		if (!this._created) 
		{
			this.onCreate(position);
		}
		else
		{
			position();
		}
		
		this.setZIndex({topmost: true});
		this.titleLayer.setZIndex({above: this});

		this.openned = true;

		this.invokeEvent('open', new DynEvent('open',this));
	}

	/**
	 * Method: close
	 *
	 *	Hides the window
	 *
	 */
	p.close = function()
	{
		this.setDisplay(false);
		this.setSize(0,0);
		thyWindow._unsetWindowPosition(this);

		this.openned = false;

		this.invokeEvent('close', new DynEvent('close',this));
	}
	
	p._thyBordePanelSetName = p.setName;
	/**
	 * Method: setName
	 *
	 *	Overwritten method that sets the name of main, title, contents and drag border
	 *	and refreshes CSS Classes
	 *
	 * Parameter:
	 *
	 *	name - The new widget name
	 *
	 */
	p.setName = function (name)
	{
		if (name == null || this.name == name) return;
		
		this._thyBorderPanelSetName(name);
		if (this.titleLayer) this.titleLayer.setName(name+'_title');
		if (this.content) this.content.setName(name+'_contents');
		if (this.btnClose) this.btnClose.setName(name+'_btnClose');
		if (this.btnStick) this.btnStick.setName(name+'_btnStick');
		if (this.winDragBorder) this.winDragBorder.setName(name+'_ondrag');
	}

	p._thyBorderPanelSetCSSClass = p.setCSSClass;
	/**
	 * Method: setCSSClass
	 *
	 *	Overwritten method that calls parent setCSSClass and also sets CSS of title,
	 *	content, buttons and drag border
	 *
	 * Parameter:
	 *
	 *	type - The class type
	 *
	 */
	p.setCSSClass = function (type)
	{
		this._thyBorderPanelSetCSSClass(type);
		if (this.titleLayer) this.titleLayer.setCSSClass(type);
		if (this.content) this.content.setCSSClass(type);
		if (this.btnClose) this.btnClose.setCSSClass(type);
		if (this.btnStick) this.btnStick.setCSSClass(type);
		if (this.winDragBorder) this.winDragBorder.setCSSClass(type);
	}
	
	p._thyBorderPanelAddChild = p.addChild;
	/**
	 * Method: addChild
	 *
	 *	Overwritten AddChild method that insert childs in contents
	 *
	 */
	p.addChild = function (c,alias,inlineID)
	{
		this.content.addChild(c,alias,inlineID);
	}

	p._thyBorderPanelRemoveChild = p.removeChild;
	/**
	 * Method: removeChild
	 *
	 * Overwritten RemoveChild method that removes childs from contents
	 *
	 */
	p.removeChild = function (c)
	{
		this.content.removeChild(c);
	}
	
	/**
	 * Method: getInnerHTML
	 *
	 *	Overwritten getInnerHTML that inserts Title
	 *
	 * @public
	 */
	p.getInnerHTML = function()
	{
		var html=['<table border="0" id="'+this.name+'_mainHolder'+'" cellspacing="0" cellpadding="0" border="0">',
		       '<tr><td id="'+this.name+'_t'+'" class="'+this.getCSSClasses('top')+'" colspan="3"></td></tr>',
		       '<tr><td id="'+this.name+'_l'+'" class="'+this.getCSSClasses('left')+'"></td>',
		       '<td id="'+this.name+'_contentHolder" class="'+this.getCSSClasses('window')+'">',// style="position: relative">'+this._thyBorderPanelGetInnerHTML(),
			   '<div style="position: relative;">'+this._thyPanelGetInnerHTML()+'</div>',
			   '</td>',
		       '<td id="'+this.name+'_r'+'" class="'+this.getCSSClasses('right')+'"></td></tr>',
		       '<tr><td id="'+this.name+'_b'+'" class="'+this.getCSSClasses('bottom')+'" colspan="3"></td></tr></table>'].join('');
		//alert(html);
		return html;
	}

	/**
	 * Method: stick
	 *
	 *	Makes window unmovable
	 *
	 */
	p.stick = function ()
	{
		if (this.sticked) return;

		this.sticked = true;
		this.btnStick.cssExtraClasses['sticked'] = {};
		this.btnStick.cssExtraClasses['sticked'][this.btnStick.cssActualSelection] = ['sticked'];
		this.btnStick.setCSSClass(this.btnStick.cssActualSelection);
		this.setDragEnabled(false);
	}

	/**
	 * Method: unstick
	 *
	 *	Release window for movement
	 *
	 */
	p.unstick = function ()
	{
		if (!this.sticked) return;

		this.sticked = false;
		delete this.btnStick.cssExtraClasses['sticked'];
		this.btnStick.setCSSClass(this.btnStick.cssActualSelection);
		this.setDragEnabled(true,null,true);
	}

	/**
	 * Method: switchStick
	 *
	 *	Change from one sticked state to another
	 *
	 */
	p.switchStick = function ()
	{
		if (this.sticked) this.unstick();
		else this.stick();
	}
	
	/*************************************************************************\
	 *                          Private Methods                              *
	\*************************************************************************/

	thyWindow._positions_x = new Object();
	thyWindow._positions_y = new Object();

	/**
	 * Method: _getWindowPosition
	 *
	 *	STATIC method that returns the next position to be used
	 *	by a window when opened
	 *
	 * Parameter:
	 *
	 *	thyWin - The thyWindow object that will be opened
	 *
	 */
	thyWindow._getWindowPosition = function (thyWin)
	{
		var positions_x = thyWindow._positions_x;
		var positions_y = thyWindow._positions_y;
		
		var w = dynapi.document.getWidth();
		var h = dynapi.document.getHeight();

		var sx = dynapi.document.getXScroll();
		var sy = dynapi.document.getYScroll();

		var ww = thyWin.content.getWidth();
		var wh = thyWin.content.getHeight();

		var x = w/2 + sx - ww/2;
		var y = h/2 + sy - wh/2;

		var i;
		
		for (i in positions_x)
		{
			while (positions_x[i][x])
			{
				x += 10;
			}
		}

		for (i in positions_y)
		{
			while (positions_y[i][y])
			{
				y += 10;
			}
		}

		if (!positions_x[thyWin.name])
		{
			positions_x[thyWin.name] = new Object();
			positions_y[thyWin.name] = new Object();
		}
		
		positions_x[thyWin.name][x] = x;
		positions_y[thyWin.name][y] = y;

		return {x:x,y:y};
	}

	/**
	 * Method: _unsetWindowPosition
	 *
	 *	STATIC method that removes the position from the list
	 *
	 */
	thyWindow._unsetWindowPosition = function (thyWin)
	{
		var positions_x = thyWindow._positions_x;
		var positions_y = thyWindow._positions_y;

		if (positions_x[thyWin.name])
		{
			delete positions_x[thyWin.name];
		}
		
		if (positions_y[thyWin.name])
		{
			delete positions_y[thyWin.name];
		}
	}

	p._thyBorderPanel_destroy = p._destroy;
	/**
	 * Method: _destroy
	 *
	 *	Overloaded method that removes winBorderPanel from memory
	 *
	 */
	p._destroy = function ()
	{
		this.winDragBorder._destroy();
		this._thyBorderPanel_destroy();
	}
	
	p._thyBorderPanel_addCSSClass = p._addCSSClass;
	/**
	 * Method: _addCSSClass
	 *
	 *	Overwritten method that adds the base class to main, title, contents, drag border
	 *	and buttons
	 *
	 * Paramenter:
	 *
	 *	cssClass - The base class string
	 *
	 */
	p._addCSSClass = function (cssClass)
	{
		this._thyBorderPanel_addCSSClass(cssClass);
		if (this.titleLayer) 
		{
			this.titleLayer._addCSSClass(cssClass+'_title');
			this.content._addCSSClass(cssClass+'_contents');
			this.btnClose._addCSSClass(cssClass+'_btnClose');
			this.btnStick._addCSSClass(cssClass+'_btnStick');
			this.winDragBorder._addCSSClass(cssClass+'_ondrag');
		}
	}
	
	p._thyBorderPanel_removeCSSClass = p._removeCSSClass;
	/**
	 * Method: _removeCSSClass
	 *
	 *	Removes the specified CSS Class from main, title, contents, drag border
	 *	and buttons
	 *
	 * Parameter:
	 *
	 *	cssClass - The CSS Class to be removes
	 *
	 */
	p._removeCSSClass = function (cssClass)
	{
		this._thyBorderPanel_removeCSSClass(cssClass);
		if (this.titleLayer) 
		{
			this.titleLayer._removeCSSClass(cssClass+'_title');
			this.content._removeCSSClass(cssClass+'_contents');
			this.btnClose._removeCSSClass(cssClass+'_btnClose');
			this.btnStick._removeCSSClass(cssClass+'_btnStick');
			this.winDragBorder._removeCSSClass(cssClass+'_ondrag');
		}
	}

	/**
	 * Method: ondragend
	 *
	 *	OnDragEnd Event. It switches the drag DynLayer to the Window Layer
	 *
	 */
	p.ondragend = function(e)
	{
		this.dragging = false;
		this.winDragBorder.setDisplay(true);
		this.setX(this.winDragBorder.getAbsoluteX());
		this.setY(this.winDragBorder.getAbsoluteY());
		this.winDragBorder.setDisplay(false);
		this.setDisplay(true);
	}

	/**
	 * Method: ondragmove
	 *
	 *	OnDragMove Event. It updates the drag layer position
	 *
	 */
	p.ondragmove = function(e)
	{
		if (this.dragging) return;
		this.setDisplay(false);
		this.dragging = true;
	}

	/**
	 * Method: onmousedown
	 *
	 *	OnMouseDown Event. Change z-index of drag layer and window.
	 *
	 */
	p.onmousedown = function(e)
	{
		this.setZIndex({topmost: true});
		this.winDragBorder.setZIndex({below: this});
	}
